@@ -12,18 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# enable x64 on jax
-# must be done at startup.
-from jax.config import config
-
-config.update("jax_enable_x64", True)
-del config
-
 from ._version import version as __version__  # noqa: F401
 
-from . import utils
 from .utils import config
 
+from . import utils
+from . import errors
+
 __all__ = [
     "exact",
     "graph",
@@ -38,6 +33,7 @@ __all__ = [
     "nn",
 ]
 
+
 from . import jax
 from . import stats
 
