@@ -14,25 +14,39 @@
 
 import time
 
+from netket.utils import struct
 
-class Timeout:
-    """A simple callback to stop NetKet after some time has passed."""
 
-    def __init__(self, timeout):
+class Timeout(struct.Pytree, mutable=True):
+    """A simple callback to stop NetKet after some time has passed.
+
+    This callback monitors whether a driver has been training for more
+    than a given timeout in order to hard stop training.
+    """
+
+    timeout: float
+    """Number of seconds to wait before the training will be stopped."""
+
+    # caches
+    _init_time: float | None = struct.field(serialize=False)
+    """
+    Internal field storing the time at which the first iteration has been
+    performed.
+    """
+
+    def __init__(self, timeout: float):
         """
-        Constructs a new Timeout object that monitors whether a driver has been training
-        for more than a given timeout in order to hard stop training.
+        Stops the optimisation after a certain time itnerval.
 
         Args:
-            timeout: Number of seconds to wait before hard stopping training.
+            timeout: number of seconds after which the optimisation will
+                be stopped.
         """
-        assert timeout > 0
-        self.__timeout = timeout
-        self.__init_time = None
+        if not timeout > 0:
+            raise ValueError("`timeout` must be larger than 0.")
+        self.timeout = timeout
 
-    def reset(self):
-        """Resets the initial time of the training"""
-        self.__init_time = None
+        self._init_time = None
 
     def __call__(self, step, log_data, driver):
         """
@@ -49,10 +63,10 @@ class Timeout:
         Note:
             This callback does not make use of `step`, `log_data` nor `driver`.
         """
-        if self.__init_time is None:
-            self.__init_time = time.time()
+        if self._init_time is None:
+            self._init_time = time.time()
+
+        if time.time() - self._init_time >= self.timeout:
+            return False
         else:
-            print(time.time() - self.__init_time)
-            if time.time() - self.__init_time >= self.__timeout:
-                return False
-        return True
+            return True
