@@ -12,10 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+
 import numpy as _np
 from scipy.sparse.linalg import bicgstab as _bicgstab
+from scipy.sparse.linalg import LinearOperator as _LinearOperator
 
 from .operator import AbstractOperator as _AbstractOperator
+from jax.experimental.sparse import JAXSparse as _JAXSparse
 
 
 def lanczos_ed(
@@ -24,7 +27,7 @@ def lanczos_ed(
     k: int = 1,
     compute_eigenvectors: bool = False,
     matrix_free: bool = False,
-    scipy_args: dict = None,
+    scipy_args: dict | None = None,
 ):
     r"""Computes `first_n` smallest eigenvalues and, optionally, eigenvectors
     of a Hermitian operator using :meth:`scipy.sparse.linalg.eigsh`.
@@ -68,10 +71,23 @@ def lanczos_ed(
     actual_scipy_args["k"] = k
     actual_scipy_args["return_eigenvectors"] = compute_eigenvectors
 
-    result = eigsh(
-        operator.to_linear_operator() if matrix_free else operator.to_sparse(),
-        **actual_scipy_args,
-    )
+    if matrix_free:
+        # wrap the operator.to_linear_operator() in a scipy.sparse.linalg.LinearOperator
+        n = operator.hilbert.n_states
+        A = _LinearOperator(
+            (n, n),
+            operator.to_linear_operator().__matmul__,
+            dtype=operator.dtype,
+        )
+    else:
+        A = operator.to_sparse()
+        if isinstance(A, _JAXSparse):
+            # jax sparse arrays are not compatible with scipy eigsh.
+            # wrap them in a scipy.sparse.linalg.LinearOperator
+            A = _LinearOperator(A.shape, A.__matmul__, dtype=A.dtype)
+
+    result = eigsh(A, **actual_scipy_args)
+
     if not compute_eigenvectors:
         # The sort order of eigenvalues returned by scipy changes based on
         # `return_eigenvalues`. Therefore we invert the order here so that the
@@ -161,7 +177,9 @@ def steady_state(lindblad, *, sparse=True, method="ed", rho0=None, **kwargs):
                 since the update to numpy 1.9.
                 We suggest using sparse=True, however, if you wish not to, you have
                 been warned.
-                Your digits are your responsibility now."""
+                Your digits are your responsibility now.
+                """,
+                stacklevel=2,
             )
 
             lind_mat = lindblad.to_dense()
