@@ -12,14 +12,28 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .metropolis_pmap import MetropolisSamplerPmap
+from . import rules
 
-from .metropolis_pt import (
-    MetropolisPtSampler,
-    MetropolisLocalPt,
-    MetropolisExchangePt,
+
+# Deprecated bindings from stabilisaation
+from netket.sampler import (
+    MetropolisFermionHop as _deprecated_MetropolisParticleExchange,
 )
 
+_deprecations = {
+    # June 2024, NetKet 3.13
+    "MetropolisParticleExchange": (
+        "netket.experimental.sampler.MetropolisParticleExchange is deprecated: use "
+        "netket.sampler.MetropolisFermionHop (netket >= 3.13)",
+        _deprecated_MetropolisParticleExchange,
+    ),
+}
+
+
+from netket.utils.deprecation import deprecation_getattr as _deprecation_getattr
 from netket.utils import _hide_submodules
 
+__getattr__ = _deprecation_getattr(__name__, _deprecations)
 _hide_submodules(__name__)
+
+del _deprecation_getattr
