@@ -31,6 +31,7 @@ from .common_lattices import (
     KitaevHoneycomb,
 )
 
-from netket.utils import _hide_submodules
+from netket.utils import _hide_submodules, _auto_export
 
+_auto_export(__name__)
 _hide_submodules(__name__)
