@@ -14,13 +14,14 @@
 
 import abc
 
-from typing import Tuple, Optional, Union, Iterable
+from typing import Union
+from collections.abc import Sequence
+from typing import Self
+
+from netket.utils.types import PRNGKeyT
+
 
 import jax.numpy as jnp
-import numpy as np
-
-max_states = np.iinfo(np.int32).max
-"""int: Maximum number of states that can be indexed"""
 
 
 class AbstractHilbert(abc.ABC):
@@ -43,14 +44,15 @@ class AbstractHilbert(abc.ABC):
     @abc.abstractmethod
     def size(self) -> int:
         r"""The number number of degrees of freedom in the basis of this
-        Hilbert space."""
+        Hilbert space.
+        """
         raise NotImplementedError()  # pragma: no cover
 
     def random_state(
         self,
-        key=None,
-        size: Optional[int] = None,
-        dtype=np.float32,
+        key: PRNGKeyT = None,
+        size: int | None = None,
+        dtype=None,
     ) -> jnp.ndarray:
         r"""Generates either a single or a batch of uniformly distributed random states.
         Runs as :code:`random_state(self, key, size=None, dtype=np.float32)` by default.
@@ -74,16 +76,16 @@ class AbstractHilbert(abc.ABC):
             >>> hi = netket.hilbert.Qubit(N=2)
             >>> k1, k2 = jax.random.split(jax.random.PRNGKey(1))
             >>> print(hi.random_state(key=k1))
-            [1. 0.]
+            [0 0]
             >>> print(hi.random_state(key=k2, size=2))
-            [[0. 0.]
-             [0. 1.]]
+            [[0 0]
+             [0 0]]
         """
         from netket.hilbert import random
 
         return random.random_state(self, key, size, dtype=dtype)
 
-    def ptrace(self, sites: Union[int, Iterable]) -> "AbstractHilbert":
+    def ptrace(self, sites: int | Sequence[int]) -> Self | None:
         """Returns the hilbert space without the selected sites.
 
         Not all hilbert spaces support this operation.
@@ -104,19 +106,62 @@ class AbstractHilbert(abc.ABC):
 
     @property
     @abc.abstractmethod
-    def _attrs(self) -> Tuple:
+    def _attrs(self) -> tuple:
         """
         Tuple of hashable attributes, used to compute the immutable
         hash of this Hilbert space
         """
 
+    def __mul__(
+        self, other: "AbstractHilbert"
+    ) -> Union[Self, "TensorGenericHilbert"]:  # noqa: F821
+        if not isinstance(other, AbstractHilbert):
+            return NotImplemented
+
+        if type(self) == type(other):
+            res = self._mul_sametype_(other)  # type: ignore
+            if res is not NotImplemented:
+                return res
+
+        from .tensor_hilbert import TensorGenericHilbert
+
+        return TensorGenericHilbert(self, other)
+
+    def __rmul__(
+        self, other: "AbstractHilbert"
+    ) -> Union[Self, "TensorGenericHilbert"]:  # noqa: F821
+        if not isinstance(other, AbstractHilbert):
+            return NotImplemented
+
+        from .tensor_hilbert import TensorHilbert
+
+        return TensorHilbert(other, self)  # type: ignore[return-value]
+
+    def _mul_sametype_(self, other: Self) -> Self:
+        """This function can be implemented by subclasses to
+        specify how to multiply two Hilbert spaces of the same type.
+
+        This can be used as an optimization to avoid creating a TensorHilbert
+        object when possible, instead returning a new Hilbert space type.
+
+        If it is not possible to combine the two Hilbert spaces,
+        it should return NotImplemented.
+
+        Args:
+            other: other Hilbert space to combine with.
+
+        Returns:
+            An Hilbert space combining the two.
+        """
+        return NotImplemented
+
     def __eq__(self, other) -> bool:
         if isinstance(other, type(self)):
             return self._attrs == other._attrs
 
         return False
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         if self._hash is None:
             self._hash = hash(self._attrs)
 
