@@ -12,10 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Optional, Union, List
+from collections.abc import Sequence
+from typing import Self
 
 import numpy as np
 
+from netket.utils import StaticRange
+
 from .homogeneous import HomogeneousHilbert
 
 
@@ -36,10 +39,7 @@ class Qubit(HomogeneousHilbert):
             >>> print(hi.size)
             100
         """
-        super().__init__([0.0, 1.0], N)
-
-    def states_to_local_indices(self, x):
-        return x.astype(np.int32)
+        super().__init__(StaticRange(0, 1, 2, dtype=np.int8), N)  # type: ignore[arg-type]
 
     def __pow__(self, n):
         return Qubit(self.size * n)
@@ -48,7 +48,7 @@ class Qubit(HomogeneousHilbert):
         assert type(self) == type(other)
         return Qubit(self.size + other.size)
 
-    def ptrace(self, sites: Union[int, List]) -> Optional["Qubit"]:
+    def ptrace(self, sites: int | Sequence[int]) -> Self | None:
         if isinstance(sites, int):
             sites = [sites]
 
@@ -66,7 +66,7 @@ class Qubit(HomogeneousHilbert):
             return Qubit(N=self.size - Nsites)
 
     def __repr__(self):
-        return "Qubit(N={})".format(self.size)
+        return f"Qubit(N={self.size})"
 
     @property
     def _attrs(self):
