@@ -69,7 +69,7 @@ def flip_state_batch(hilb: Fock, key, σ, idx):
 
 from netket.utils import _hide_submodules
 
-from . import custom, doubled, fock, qubit, spin, tensor_hilbert, particle
+from . import doubled, homogeneous, fock, qubit, tensor_hilbert
 from .base import flip_state, random_state
 
 _hide_submodules(__name__)
