@@ -12,14 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from .base import AbstractLog
 from .runtime_log import RuntimeLog
 from .json_log import JsonLog
 from .state_log import StateLog
 from .tensorboard import TensorBoardLog
 
-from netket.utils import tensorboard_available as _tensorboard_available
 
+from netket.utils import _hide_submodules, _auto_export
 
-from netket.utils import _hide_submodules
-
+_auto_export(__name__)
 _hide_submodules(__name__)
