@@ -19,6 +19,8 @@ from os import path as _path
 
 from flax import serialization
 
+from netket.jax.sharding import extract_replicated
+
 from .runtime_log import RuntimeLog
 
 
@@ -120,11 +122,13 @@ class JsonLog(RuntimeLog):
         self._autoflush_cost = autoflush_cost
         self._last_flush_time = time.time()
         self._last_flush_runtime = 0.0
+        self._last_flush_pars_time = time.time()
+        self._last_flush_pars_runtime = 0.0
 
         self._flush_log_time = 0.0
         self._flush_pars_time = 0.0
 
-    def __call__(self, step, item, variational_state):
+    def __call__(self, step, item, variational_state=None):
         old_step = self._old_step
         super().__call__(step, item, variational_state)
 
@@ -144,9 +148,16 @@ class JsonLog(RuntimeLog):
             or flush_anyway
         ):
             self._flush_log()
+
+        elapsed_time = time.time() - self._last_flush_pars_time
+        flush_anyway = (
+            self._last_flush_pars_runtime / (elapsed_time + 1e-7) < self._autoflush_cost
+        )
+
         if (
             self._steps_notflushed_pars % self._save_params_every == 0
             or step == old_step - 1
+            or flush_anyway
         ):
             self._flush_params(variational_state)
 
@@ -157,7 +168,8 @@ class JsonLog(RuntimeLog):
     def _flush_log(self):
         # Time how long flushing data takes.
         self._last_flush_time = time.time()
-        self.serialize(self._prefix + ".log")
+        if self._is_master_process:
+            self.serialize(self._prefix + ".log")
         self._last_flush_runtime = time.time() - self._last_flush_time
 
         self._flush_log_time += self._last_flush_runtime
@@ -166,15 +178,20 @@ class JsonLog(RuntimeLog):
     def _flush_params(self, variational_state):
         if not self._save_params:
             return
+        if variational_state is None:
+            return
 
-        _time = time.time()
-
-        binary_data = serialization.to_bytes(variational_state.variables)
-        with open(self._prefix + ".mpack", "wb") as outfile:
-            outfile.write(binary_data)
+        self._last_flush_pars_time = time.time()
+        binary_data = serialization.to_bytes(
+            extract_replicated(variational_state.variables)
+        )
+        if self._is_master_process:
+            with open(self._prefix + ".mpack", "wb") as outfile:
+                outfile.write(binary_data)
+        self._last_flush_pars_runtime = time.time() - self._last_flush_pars_time
 
+        self._flush_pars_time += self._last_flush_pars_runtime
         self._steps_notflushed_pars = 0
-        self._flush_pars_time += time.time() - _time
 
     def flush(self, variational_state=None):
         """
@@ -188,6 +205,14 @@ class JsonLog(RuntimeLog):
         if variational_state is not None:
             self._flush_params(variational_state)
 
+    def __del__(self):
+        if hasattr(self, "_steps_notflushed_write"):
+            if self._steps_notflushed_write > 0:
+                self.flush()
+        if hasattr(self, "_steps_notflushed_pars"):
+            if self._steps_notflushed_pars > 0 and self._save_params:
+                self.flush()
+
     def __repr__(self):
         _str = f"JsonLog('{self._prefix}', mode={self._file_mode}, "
         _str = _str + f"autoflush_cost={self._autoflush_cost})"
