@@ -12,17 +12,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Union, IO
+from typing import Any, IO, TYPE_CHECKING
 from pathlib import Path
 
 import jax
 import numpy as np
 import orjson
 
-from netket.utils import accum_histories_in_tree
+from netket.utils.history import accum_histories_in_tree, HistoryDict
+
+if TYPE_CHECKING:
+    from netket.vqs import VariationalState
+
+from .base import AbstractLog
 
 
-class RuntimeLog:
+class RuntimeLog(AbstractLog):
     """
     This logger accumulates log data in a set of nested dictionaries which are stored in memory. The log data is not automatically saved to the filesystem.
 
@@ -33,31 +38,40 @@ class RuntimeLog:
     the current content to a file, use the method :py:meth:`~netket.logging.RuntimeLog.serialize`.
     """
 
+    _data: dict[str, Any]
+
     def __init__(self):
         """
         Crates a Runtime Logger.
         """
-        self._data = None
+        self._data: dict[str, Any] = HistoryDict()
         self._old_step = 0
 
-    def __call__(self, step, item, variational_state):
+    def __call__(
+        self,
+        step: int,
+        item: dict[str, Any],
+        variational_state: "VariationalState | None" = None,
+    ):
+        if self._data is None:
+            self._data = {}
         self._data = accum_histories_in_tree(self._data, item, step=step)
         self._old_step = step
 
     @property
-    def data(self):
+    def data(self) -> dict[str, Any]:
         """
         The dictionary of logged data.
         """
         return self._data
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> Any:
         return self.data[key]
 
-    def flush(self, variational_state):
+    def flush(self, variational_state=None):
         pass
 
-    def serialize(self, path: Union[str, Path, IO]):
+    def serialize(self, path: str | Path | IO):
         r"""
         Serialize the content of :py:attr:`~netket.logging.RuntimeLog.data` to a file.
 
@@ -69,6 +83,9 @@ class RuntimeLog:
         if isinstance(path, str):
             path = Path(path)
 
+        if not self._is_master_process:
+            return
+
         if isinstance(path, Path):
             parent = path.parent
             filename = path.name
@@ -95,7 +112,8 @@ class RuntimeLog:
 
     def __repr__(self):
         _str = "RuntimeLog():\n"
-        _str += f" keys = {list(self.data.keys())}"
+        if self.data is not None:
+            _str += f" keys = {list(self.data.keys())}"
         return _str
 
 
@@ -110,6 +128,8 @@ def default(obj):
                 "real": np.ascontiguousarray(obj.real),
                 "imag": np.ascontiguousarray(obj.imag),
             }
+        else:
+            return np.ascontiguousarray(obj)
     elif isinstance(obj, jax.numpy.ndarray):
         return np.ascontiguousarray(obj)
     elif isinstance(obj, complex):
