@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import Any, Literal, Union, TYPE_CHECKING
+
 import tarfile
 import time
 from io import BytesIO
@@ -22,6 +24,22 @@ from os import path as _path
 
 from flax import serialization
 
+from netket.jax.sharding import extract_replicated
+
+if TYPE_CHECKING:
+    from netket.vqs import VariationalState
+
+from .base import AbstractLog
+
+FileModeT = Union[
+    Literal["write"],
+    Literal["append"],
+    Literal["fail"],
+    Literal["x"],
+    Literal["w"],
+    Literal["a"],
+]
+
 
 def save_binary_to_tar(tar_file, byte_data, name):
     abuf = BytesIO(byte_data)
@@ -34,7 +52,7 @@ def save_binary_to_tar(tar_file, byte_data, name):
     tar_file.addfile(tarinfo=info, fileobj=abuf)
 
 
-class StateLog:
+class StateLog(AbstractLog):
     """
     A logger which serializes the variables of the variational state during a run.
 
@@ -47,7 +65,7 @@ class StateLog:
     def __init__(
         self,
         output_prefix: str,
-        mode: str = "write",
+        mode: FileModeT = "write",
         save_every: int = 1,
         tar: bool = False,
     ):
@@ -101,7 +119,7 @@ class StateLog:
             os.makedirs(dir_name, exist_ok=True)
 
         self._prefix = output_prefix
-        self._file_mode = mode
+        self._file_mode: FileModeT = mode
 
         self._save_every = save_every
         self._old_step = 0
@@ -115,16 +133,19 @@ class StateLog:
         self._tar_file = None
         self._closed = False
 
+        self._file_step = 0
+
     def _init_output(self):
-        if self._tar:
-            self._create_tar_file()
-        else:
-            self._check_output_folder()
+        if self._is_master_process:
+            if self._tar:
+                self._create_tar_file()
+            else:
+                self._check_output_folder()
         self._init = True
 
     def _create_tar_file(self):
         if self._tar_file is None:
-            self._tar_file = tarfile.TarFile(self._prefix + ".tar", self._file_mode[0])
+            self._tar_file = tarfile.TarFile(self._prefix + ".tar", self._file_mode[0])  # type: ignore
             self._file_step = 0
             if self._file_mode == "append":
                 files = self._tar_file.getnames()
@@ -149,7 +170,12 @@ class StateLog:
             self._tar_file.close()
             self._closed = True
 
-    def __call__(self, step, item, variational_state):
+    def __call__(
+        self,
+        step: int,
+        item: dict[str, Any],
+        variational_state: "VariationalState | None" = None,
+    ):
         old_step = self._old_step
 
         if self._steps_notsaved % self._save_every == 0 or step == old_step - 1:
@@ -163,14 +189,18 @@ class StateLog:
             self._init_output()
 
         _time = time.time()
-        binary_data = serialization.to_bytes(variational_state.variables)
-        if self._tar:
-            save_binary_to_tar(
-                self._tar_file, binary_data, str(self._file_step) + ".mpack"
-            )
-        else:
-            with open(self._prefix + str(self._file_step) + ".mpack", "wb") as f:
-                f.write(binary_data)
+        binary_data = serialization.to_bytes(
+            extract_replicated(variational_state.variables)
+        )
+
+        if self._is_master_process:
+            if self._tar:
+                save_binary_to_tar(
+                    self._tar_file, binary_data, str(self._file_step) + ".mpack"
+                )
+            else:
+                with open(self._prefix + str(self._file_step) + ".mpack", "wb") as f:
+                    f.write(binary_data)
 
         self._file_step += 1
         self._runtime_taken += time.time() - _time
@@ -179,7 +209,7 @@ class StateLog:
         if hasattr(self, "_closed"):
             self.close()
 
-    def flush(self, variational_state):
+    def flush(self, variational_state=None):
         pass
 
     def __repr__(self):
