@@ -1,4 +1,4 @@
-# Copyright 2021 The NetKet Authors - All rights reserved.
+# Copyright 2021-2024 The NetKet Authors - All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,31 +12,35 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from . import tensor_networks
+
 from .rbm import RBM, RBMModPhase, RBMMultiVal, RBMSymm
 from .equivariant import GCNN
 from .full_space import LogStateVector
 from .jastrow import Jastrow
-from .mps import MPSPeriodic
 from .gaussian import Gaussian
 from .deepset import DeepSetRelDistance, DeepSetMLP
 from .ndm import NDM
-from .autoreg import AbstractARNN, ARNNDense, ARNNConv1D, ARNNConv2D as _ARNNConv2D
+from .autoreg import (
+    AbstractARNN,
+    ARNNSequential,
+    ARNNDense,
+    ARNNConv1D,
+    ARNNConv2D,
+)
 from .fast_autoreg import (
+    FastARNNSequential,
     FastARNNDense,
     FastARNNConv1D,
-    FastARNNConv2D as _FastARNNConv2D,
+    FastARNNConv2D,
 )
 from .mlp import MLP
+
+from .slater import Slater2nd, MultiSlater2nd
+
 from .utils import update_GCNN_parity
 
 
-# TODO: remove dtype deprecation
-from netket.utils import deprecate_dtype as _deprecate_dtype
-
-ARNNConv2D = _deprecate_dtype(_ARNNConv2D)
-FastARNNConv2D = _deprecate_dtype(_FastARNNConv2D)
-
-
 from netket.utils import _hide_submodules
 
 _hide_submodules(__name__)
