@@ -1,4 +1,5 @@
-from typing import Union, Tuple, Optional, Callable
+from collections.abc import Callable
+from typing import TYPE_CHECKING
 
 import jax
 from jax import numpy as jnp
@@ -10,10 +11,11 @@ from jax.nn.initializers import (
     lecun_normal,
 )
 
-from netket.utils import deprecate_dtype
-from netket.hilbert import ContinuousHilbert
 import netket.nn as nknn
 
+if TYPE_CHECKING:
+    from netket.experimental.hilbert import ContinuousHilbert
+
 
 class DeepSetMLP(nn.Module):
     r"""Implements the DeepSets architecture, which is permutation invariant.
@@ -30,12 +32,12 @@ class DeepSetMLP(nn.Module):
     https://arxiv.org/abs/1703.06114
     """
 
-    features_phi: Optional[Union[int, Tuple[int, ...]]] = None
+    features_phi: int | tuple[int, ...] | None = None
     """
     Number of features in each layer for phi network.
     When features_phi is None, no phi network is created.
     """
-    features_rho: Optional[Union[int, Tuple[int, ...]]] = None
+    features_rho: int | tuple[int, ...] | None = None
     """
     Number of features in each layer for rho network.
     Should not include the final layer of dimension 1, which is included automatically.
@@ -45,9 +47,9 @@ class DeepSetMLP(nn.Module):
     param_dtype: DType = jnp.float64
     """The dtype of the weights."""
 
-    hidden_activation: Optional[Callable] = jax.nn.gelu
+    hidden_activation: Callable | None = jax.nn.gelu
     """The nonlinear activation function between hidden layers."""
-    output_activation: Optional[Callable] = None
+    output_activation: Callable | None = None
     """The nonlinear activation function at the output layer."""
 
     pooling: Callable = jnp.sum
@@ -60,8 +62,8 @@ class DeepSetMLP(nn.Module):
     """Initializer for the Dense layer matrix"""
     bias_init: NNInitFunc = zeros
     """Initializer for the hidden bias"""
-    precision: Optional[jax.lax.Precision] = None
-    """numerical precision of the computation see `jax.lax.Precision`for details."""
+    precision: jax.lax.Precision | None = None
+    """numerical precision of the computation see :class:`jax.lax.Precision` for details."""
 
     @nn.compact
     def __call__(self, input):
@@ -91,10 +93,12 @@ def _process_features_rho(features_input):
     else:
         if not hasattr(features_input, "__len__"):
             raise ValueError("features_rho must be a sequence of integers")
-        return tuple(features_input) + (1,)
+        return (
+            *tuple(features_input),
+            1,
+        )
 
 
-@deprecate_dtype
 class DeepSetRelDistance(nn.Module):
     r"""Implements an equivariant version of the DeepSets architecture
     given by (https://arxiv.org/abs/1703.06114)
@@ -114,7 +118,7 @@ class DeepSetRelDistance(nn.Module):
 
     """
 
-    hilbert: ContinuousHilbert
+    hilbert: "ContinuousHilbert"
     """The hilbert space defining the periodic box where this ansatz is defined."""
 
     layers_phi: int
@@ -122,23 +126,23 @@ class DeepSetRelDistance(nn.Module):
     layers_rho: int
     """Number of layers in rho network."""
 
-    features_phi: Union[Tuple, int]
+    features_phi: tuple | int
     """Number of features in each layer for phi network."""
-    features_rho: Union[Tuple, int]
+    features_rho: tuple | int
     """
     Number of features in each layer for rho network.
     If specified as a list, the last layer must have 1 feature.
     """
 
-    cusp_exponent: Optional[int] = None
+    cusp_exponent: int | None = None
     """exponent of Katos cusp condition"""
 
     param_dtype: DType = jnp.float64
     """The dtype of the weights."""
 
-    activation: Optional[Callable] = jax.nn.gelu
+    activation: Callable | None = jax.nn.gelu
     """The nonlinear activation function between hidden layers."""
-    output_activation: Optional[Callable] = None
+    output_activation: Callable | None = None
     """The nonlinear activation function at the output layer."""
 
     pooling: Callable = jnp.sum
@@ -155,8 +159,7 @@ class DeepSetRelDistance(nn.Module):
     """Initializer for the parameter in the cusp"""
 
     def setup(self):
-
-        if not all(self.hilbert.pbc):
+        if not all(self.hilbert.geometry.pbc):
             raise ValueError(
                 "The DeepSetRelDistance model only works with "
                 "hilbert spaces with periodic boundary conditions "
@@ -205,7 +208,7 @@ class DeepSetRelDistance(nn.Module):
         batch_shape = x.shape[:-1]
         param = self.param("cusp", self.params_init, (1,), self.param_dtype)
 
-        L = jnp.array(self.hilbert.extent)
+        L = jnp.array(self.hilbert.domain)
         sdim = L.size
 
         d = jax.vmap(self.distance, in_axes=(0, None, None))(x, sdim, L)
