@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Tuple, Any
+from typing import Any
 
 import numpy as np
 
@@ -22,7 +22,7 @@ from flax import linen as nn
 from jax.nn.initializers import zeros, lecun_normal
 from jax.scipy.special import logsumexp
 
-from netket.utils import HashableArray, warn_deprecation, deprecate_dtype
+from netket.utils import HashableArray
 from netket.utils.types import NNInitFunc, Array
 from netket.utils.group import PermutationGroup
 from netket.graph import Graph, Lattice
@@ -44,7 +44,6 @@ def identity(x):
     return x
 
 
-@deprecate_dtype
 class GCNN_FFT(nn.Module):
     r"""Implements a GCNN using a fast fourier transform over the translation group.
 
@@ -61,11 +60,11 @@ class GCNN_FFT(nn.Module):
     """Product table describing the algebra of the symmetry group
     Numpy/Jax arrays must be wrapped into an :class:`netket.utils.HashableArray`.
     """
-    shape: Tuple
+    shape: tuple
     """Shape of the translation group"""
     layers: int
     """Number of layers (not including sum layer over output)."""
-    features: Tuple
+    features: tuple
     """Number of features in each layer starting from the input. If a single number is given,
     all layers will have the same number of features."""
     characters: HashableArray
@@ -99,7 +98,6 @@ class GCNN_FFT(nn.Module):
     `characters` are negative."""
 
     def setup(self):
-
         self.n_symm = np.asarray(self.symmetries).shape[0]
 
         self.dense_symm = DenseSymmFFT(
@@ -152,7 +150,6 @@ class GCNN_FFT(nn.Module):
             return x
 
 
-@deprecate_dtype
 class GCNN_Irrep(nn.Module):
     r"""Implements a GCNN by projecting onto irreducible
     representations of the group. The projection onto
@@ -184,11 +181,11 @@ class GCNN_Irrep(nn.Module):
     """A group of symmetry operations (or array of permutation indices) over which the network should be equivariant.
     Numpy/Jax arrays must be wrapped into an :class:`netket.utils.HashableArray`.
     """
-    irreps: Tuple[HashableArray]
+    irreps: tuple[HashableArray]
     """List of irreducible representation matrices"""
     layers: int
     """Number of layers (not including sum layer over output)."""
-    features: Tuple
+    features: tuple
     """Number of features in each layer starting from the input. If a single number is given,
     all layers will have the same number of features."""
     characters: HashableArray
@@ -222,7 +219,6 @@ class GCNN_Irrep(nn.Module):
     `characters` are negative."""
 
     def setup(self):
-
         self.n_symm = np.asarray(self.symmetries).shape[0]
 
         self.dense_symm = DenseSymmMatrix(
@@ -273,7 +269,6 @@ class GCNN_Irrep(nn.Module):
             return x
 
 
-@deprecate_dtype
 class GCNN_Parity_FFT(nn.Module):
     r"""Implements a GCNN using a fast fourier transform over the translation group.
     The group convolution can be written in terms of translational convolutions with
@@ -290,11 +285,11 @@ class GCNN_Parity_FFT(nn.Module):
     """Product table describing the algebra of the symmetry group
     Numpy/Jax arrays must be wrapped into an :class:`netket.utils.HashableArray`.
     """
-    shape: Tuple
+    shape: tuple
     """Shape of the translation group"""
     layers: int
     """Number of layers (not including sum layer over output)."""
-    features: Tuple
+    features: tuple
     """Number of features in each layer starting from the input. If a single number is given,
     all layers will have the same number of features."""
     characters: HashableArray
@@ -319,10 +314,6 @@ class GCNN_Parity_FFT(nn.Module):
     """If true forces all basis states to have the same amplitude by setting Re[psi] = 0"""
     use_bias: bool = True
     """if True uses a bias in all layers."""
-    extra_bias: bool = False
-    """Deprecated. If True, uses bias in parity-flip layers too. Required for using
-    parameters saved before PR#1030, but hinders performance.
-    See also `nk.models.update_GCNN_parity`."""
     precision: Any = None
     """numerical precision of the computation see :class:`jax.lax.Precision` for details."""
     kernel_init: NNInitFunc = default_gcnn_initializer
@@ -334,17 +325,6 @@ class GCNN_Parity_FFT(nn.Module):
     `characters` are negative."""
 
     def setup(self):
-        # TODO: eventually remove this warning
-        # supports a deprecated attribute
-        if self.extra_bias:
-            warn_deprecation(
-                (
-                    "`extra_bias` is detrimental for performance and is deprecated. "
-                    "Please switch to the default `extra_bias=False`. Previously saved "
-                    "parameters can be migrated using `nk.models.update_GCNN_parity`."
-                )
-            )
-
         self.n_symm = np.asarray(self.symmetries).shape[0]
 
         self.dense_symm = DenseSymmFFT(
@@ -380,7 +360,7 @@ class GCNN_Parity_FFT(nn.Module):
                 shape=self.shape,
                 features=self.features[layer + 1],
                 # this would bias the same outputs as self.equivariant
-                use_bias=self.extra_bias and self.use_bias,
+                use_bias=self.use_bias,
                 param_dtype=self.param_dtype,
                 precision=self.precision,
                 kernel_init=self.kernel_init,
@@ -442,7 +422,6 @@ class GCNN_Parity_FFT(nn.Module):
             return x
 
 
-@deprecate_dtype
 class GCNN_Parity_Irrep(nn.Module):
     r"""Implements a GCNN by projecting onto irreducible
     representations of the group. The projection onto
@@ -477,11 +456,11 @@ class GCNN_Parity_Irrep(nn.Module):
     """A group of symmetry operations (or array of permutation indices) over which the network should be equivariant.
     Numpy/Jax arrays must be wrapped into an :class:`netket.utils.HashableArray`.
     """
-    irreps: Tuple[HashableArray]
+    irreps: tuple[HashableArray]
     """List of irreducible representation matrices"""
     layers: int
     """Number of layers (not including sum layer over output)."""
-    features: Tuple
+    features: tuple
     """Number of features in each layer starting from the input. If a single number is given,
     all layers will have the same number of features."""
     characters: HashableArray
@@ -506,10 +485,6 @@ class GCNN_Parity_Irrep(nn.Module):
     """If true forces all basis states to have the same amplitude by setting Re[psi] = 0"""
     use_bias: bool = True
     """if True uses a bias in all layers."""
-    extra_bias: bool = False
-    """Deprecated. If True, uses bias in parity-flip layers too. Required for using
-    parameters saved before PR#1030, but hinders performance.
-    See also `nk.models.update_GCNN_parity`."""
     precision: Any = None
     """numerical precision of the computation see :class:`jax.lax.Precision` for details."""
     kernel_init: NNInitFunc = default_gcnn_initializer
@@ -521,17 +496,6 @@ class GCNN_Parity_Irrep(nn.Module):
     `characters` are negative."""
 
     def setup(self):
-        # TODO: eventually remove this warning
-        # supports a deprecated attribute
-        if self.extra_bias:
-            warn_deprecation(
-                (
-                    "`extra_bias` is detrimental for performance and is deprecated. "
-                    "Please switch to the default `extra_bias=False`. Previously saved "
-                    "parameters can be migrated using `nk.models.update_GCNN_parity`."
-                )
-            )
-
         self.n_symm = np.asarray(self.symmetries).shape[0]
 
         self.dense_symm = DenseSymmMatrix(
@@ -564,7 +528,7 @@ class GCNN_Parity_Irrep(nn.Module):
                 irreps=self.irreps,
                 features=self.features[layer + 1],
                 # this would bias the same outputs as self.equivariant
-                use_bias=self.extra_bias and self.use_bias,
+                use_bias=self.use_bias,
                 param_dtype=self.param_dtype,
                 precision=self.precision,
                 kernel_init=self.kernel_init,
@@ -626,7 +590,6 @@ class GCNN_Parity_Irrep(nn.Module):
             return x
 
 
-@deprecate_dtype
 def GCNN(
     symmetries=None,
     product_table=None,
@@ -696,12 +659,13 @@ def GCNN(
         bias_init: Initializer for the biases of all layers.
         complex_output: If True, ensures that the network output is always complex.
             Necessary when network parameters are real but some `characters` are negative.
-        input_mask: Optional array of shape `(n_sites,)` used to restrict the convolutional
-        kernel. Only parameters with mask :math:'\ne 0' are used. For best performance a
-        boolean mask should be used.
-        hidden_mask: Optional array of shape `(n_symm,)` where `(n_symm,)` = `len(graph.automorphisms())`
-        used to restrict the convolutional kernel. Only parameters with mask :math:'\ne 0' are used.
-        For best performance a boolean mask should be used.
+        input_mask: Optional array of shape :code:`(n_sites,)` used to restrict the convolutional
+            kernel. Only parameters with mask :math:'\ne 0' are used. For best performance a
+            boolean mask should be used.
+        hidden_mask: Optional array of shape :code:`(n_symm,)` where
+            :code:`(n_symm,) = len(graph.automorphisms())` used to restrict the convolutional
+            kernel. Only parameters with mask :math:'\ne 0' are used.
+            For best performance a boolean mask should be used.
 
     """
 
