@@ -13,17 +13,17 @@
 # limitations under the License.
 
 from math import sqrt
-from typing import Any, Callable, Tuple, Union
+from typing import Any
+from collections.abc import Callable
 
 from jax import numpy as jnp
 from jax.nn.initializers import zeros
 
-from netket.models.autoreg import ARNNSequential, _normalize
+from netket.models.autoreg import ARNNSequential, _get_feature_list, _normalize
 from netket.nn import FastMaskedConv1D, FastMaskedConv2D, FastMaskedDense1D
 from netket.nn import activation as nkactivation
 from netket.nn.masked_linear import default_kernel_init
 from netket.utils.types import Array, DType, NNInitFunc
-from netket.utils import deprecate_dtype
 
 
 class FastARNNSequential(ARNNSequential):
@@ -32,6 +32,16 @@ class FastARNNSequential(ARNNSequential):
 
     Subclasses must implement `activation` as a field or a method,
     and assign a list of fast ARNN layers to `self._layers` in `setup`.
+
+    The fast autoregressive sampling is described in `Ramachandran et. {\\it al} <https://arxiv.org/abs/1704.06001>`_.
+    To generate one sample using an autoregressive network, we need to evaluate the network `N` times, where `N` is
+    the number of input sites. But actually we only change one input site each time, and not all intermediate results
+    depend on the changed input because of the autoregressive property, so we can cache unchanged intermediate results
+    and avoid repeated computation.
+
+    This optimization is particularly useful for convolutional neural networks (CNN) and recurrent neural networks (RNN)
+    where each output site of a layer only depends on a small number of input sites, while not so useful for densely
+    connected layers.
     """
 
     def conditional(self, inputs: Array, index: int) -> Array:
@@ -42,11 +52,11 @@ class FastARNNSequential(ARNNSequential):
         if inputs.ndim == 1:
             inputs = jnp.expand_dims(inputs, axis=0)
 
-        # When `index = 0`, it doesn't matter which site we take
-        x = inputs[:, index - 1, None]
+        x = jnp.expand_dims(inputs, axis=-1)
+        x = self._take_prev_site(x, index)
 
         for i in range(len(self._layers)):
-            if i > 0:
+            if i > 0 and hasattr(self, "activation"):
                 x = self.activation(x)
             x = self._layers[i].update_site(x, index)
 
@@ -54,21 +64,28 @@ class FastARNNSequential(ARNNSequential):
         p = jnp.exp(self.machine_pow * log_psi.real)
         return p
 
+    def _take_prev_site(self, inputs: Array, index: int) -> Array:
+        """
+        Takes the previous site in the autoregressive order.
+        """
+        # When `index = 0`, it doesn't matter which site we take
+        return inputs[:, index - 1]
+
 
-@deprecate_dtype
 class FastARNNDense(FastARNNSequential):
     """
     Fast autoregressive neural network with dense layers.
 
-    See :class:`netket.nn.FastMaskedConv1D` for a brief explanation of fast autoregressive sampling.
+    See :class:`netket.models.FastARNNSequential` for a brief explanation
+    of fast autoregressive sampling.
 
-    TODO: FastMaskedDense1D does not support JIT yet, because it involves slicing the cached inputs
-    and the weights with a dynamic shape.
+    TODO: FastMaskedDense1D does not support JIT yet, because it involves
+    slicing the cached inputs and the weights with a dynamic shape.
     """
 
     layers: int
     """number of layers."""
-    features: Union[Tuple[int, ...], int]
+    features: tuple[int, ...] | int
     """output feature density in each layer. If a single number is given,
     all layers except the last one will have the same number of features."""
     activation: Callable[[Array], Array] = nkactivation.reim_selu
@@ -87,13 +104,7 @@ class FastARNNDense(FastARNNSequential):
     """exponent to normalize the outputs of `__call__`."""
 
     def setup(self):
-        if isinstance(self.features, int):
-            features = [self.features] * (self.layers - 1) + [self.hilbert.local_size]
-        else:
-            features = self.features
-        assert len(features) == self.layers
-        assert features[-1] == self.hilbert.local_size
-
+        features = _get_feature_list(self)
         self._layers = [
             FastMaskedDense1D(
                 size=self.hilbert.size,
@@ -109,17 +120,17 @@ class FastARNNDense(FastARNNSequential):
         ]
 
 
-@deprecate_dtype
 class FastARNNConv1D(FastARNNSequential):
     """
     Fast autoregressive neural network with 1D convolution layers.
 
-    See :class:`netket.nn.FastMaskedConv1D` for a brief explanation of fast autoregressive sampling.
+    See :class:`netket.models.FastARNNSequential` for a brief
+    explanation of fast autoregressive sampling.
     """
 
     layers: int
     """number of layers."""
-    features: Union[Tuple[int, ...], int]
+    features: tuple[int, ...] | int
     """output feature density in each layer. If a single number is given,
     all layers except the last one will have the same number of features."""
     kernel_size: int
@@ -142,13 +153,7 @@ class FastARNNConv1D(FastARNNSequential):
     """exponent to normalize the outputs of `__call__`."""
 
     def setup(self):
-        if isinstance(self.features, int):
-            features = [self.features] * (self.layers - 1) + [self.hilbert.local_size]
-        else:
-            features = self.features
-        assert len(features) == self.layers
-        assert features[-1] == self.hilbert.local_size
-
+        features = _get_feature_list(self)
         self._layers = [
             FastMaskedConv1D(
                 features=features[i],
@@ -169,17 +174,17 @@ class FastARNNConv2D(FastARNNSequential):
     """
     Fast autoregressive neural network with 2D convolution layers.
 
-    See :class:`netket.nn.FastMaskedConv1D` for a brief explanation of fast autoregressive sampling.
+    See :class:`netket.models.FastARNNSequential` for a brief explanation of fast autoregressive sampling.
     """
 
     layers: int
     """number of layers."""
-    features: Union[Tuple[int, ...], int]
+    features: tuple[int, ...] | int
     """output feature density in each layer. If a single number is given,
     all layers except the last one will have the same number of features."""
-    kernel_size: Tuple[int, int]
+    kernel_size: tuple[int, int]
     """shape of the convolutional kernel `(h, w)`. Typically, `h = w // 2 + 1`."""
-    kernel_dilation: Tuple[int, int] = (1, 1)
+    kernel_dilation: tuple[int, int] = (1, 1)
     """a sequence of 2 integers, giving the dilation factor to
     apply in each spatial dimension of the convolution kernel (default: 1)."""
     activation: Callable[[Array], Array] = nkactivation.reim_selu
@@ -201,13 +206,7 @@ class FastARNNConv2D(FastARNNSequential):
         self.L = int(sqrt(self.hilbert.size))
         assert self.L**2 == self.hilbert.size
 
-        if isinstance(self.features, int):
-            features = [self.features] * (self.layers - 1) + [self.hilbert.local_size]
-        else:
-            features = self.features
-        assert len(features) == self.layers
-        assert features[-1] == self.hilbert.local_size
-
+        features = _get_feature_list(self)
         self._layers = [
             FastMaskedConv2D(
                 L=self.L,
