@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Union, Any
+from typing import Any
 
 import numpy as np
 
@@ -21,7 +21,7 @@ from jax import numpy as jnp
 from flax import linen as nn
 from jax.nn.initializers import normal
 
-from netket.utils import HashableArray, deprecate_dtype
+from netket.utils import HashableArray
 from netket.utils.types import NNInitFunc
 from netket.utils.group import PermutationGroup
 from netket import nn as nknn
@@ -29,7 +29,6 @@ from netket import nn as nknn
 default_kernel_init = normal(stddev=0.01)
 
 
-@deprecate_dtype
 class RBM(nn.Module):
     r"""A restricted boltzman Machine, equivalent to a 2-layer FFNN with a
     nonlinear activation function in between.
@@ -39,7 +38,7 @@ class RBM(nn.Module):
     """The dtype of the weights."""
     activation: Any = nknn.log_cosh
     """The nonlinear activation function."""
-    alpha: Union[float, int] = 1
+    alpha: float | int = 1
     """feature density. Number of features equal to alpha * input.shape[-1]"""
     use_hidden_bias: bool = True
     """if True uses a bias in the dense layer (hidden layer bias)."""
@@ -82,7 +81,6 @@ class RBM(nn.Module):
             return x
 
 
-@deprecate_dtype
 class RBMModPhase(nn.Module):
     r"""
     A fully connected Restricted Boltzmann Machine (RBM) with real-valued parameters.
@@ -105,7 +103,7 @@ class RBMModPhase(nn.Module):
     """The dtype of the weights."""
     activation: Any = nknn.log_cosh
     """The nonlinear activation function."""
-    alpha: Union[float, int] = 1
+    alpha: float | int = 1
     """feature density. Number of features equal to alpha * input.shape[-1]"""
     use_hidden_bias: bool = True
     """if True uses a bias in the dense layer (hidden layer bias)."""
@@ -144,7 +142,6 @@ class RBMModPhase(nn.Module):
         return re + 1j * im
 
 
-@deprecate_dtype
 class RBMMultiVal(nn.Module):
     """
     A fully connected Restricted Boltzmann Machine (see :class:`netket.models.RBM`) suitable for large local hilbert spaces.
@@ -159,7 +156,7 @@ class RBMMultiVal(nn.Module):
     """The dtype of the weights."""
     activation: Any = nknn.log_cosh
     """The nonlinear activation function."""
-    alpha: Union[float, int] = 1
+    alpha: float | int = 1
     """feature density. Number of features equal to alpha * input.shape[-1]"""
     use_hidden_bias: bool = True
     """if True uses a bias in the dense layer (hidden layer bias)."""
@@ -194,15 +191,21 @@ class RBMMultiVal(nn.Module):
         # do the one hot encoding: output x.shape +(n_classes,)
         x_oh = jax.nn.one_hot(x, self.n_classes)
         # vectorize the last two dimensions
-        x_oh = jnp.reshape(x_oh, batches + (self.n_classes * N,))
+        x_oh = jnp.reshape(
+            x_oh,
+            (
+                *batches,
+                self.n_classes * N,
+            ),
+        )
         # apply the rbm to this output
         return self.RBM(x_oh)
 
 
 class RBMSymm(nn.Module):
-    """A symmetrized RBM using the :ref:`netket.nn.DenseSymm` layer internally."""
+    """A symmetrized RBM using the :func:`netket.nn.DenseSymm` layer internally."""
 
-    symmetries: Union[HashableArray, PermutationGroup]
+    symmetries: HashableArray | PermutationGroup
     """A group of symmetry operations (or array of permutation indices) over which the layer should be invariant.
     Numpy/Jax arrays must be wrapped into an :class:`netket.utils.HashableArray`.
     """
@@ -210,7 +213,7 @@ class RBMSymm(nn.Module):
     """The dtype of the weights."""
     activation: Any = nknn.log_cosh
     """The nonlinear activation function."""
-    alpha: Union[float, int] = 1
+    alpha: float | int = 1
     """feature density. Number of features equal to alpha * input.shape[-1]"""
     use_hidden_bias: bool = True
     """if True uses a bias in the dense layer (hidden layer bias)."""
