@@ -13,6 +13,8 @@
 # limitations under the License.
 
 from .mlp import MLP
+from .deepset import DeepSetMLP
+from .symmetry_sum import SymmExpSum
 
 from netket.utils import _hide_submodules
 
