@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Any, Tuple
+from typing import Any
 
 from flax import linen as nn
 from flax.linen.dtypes import promote_dtype
@@ -30,15 +30,13 @@ from netket.nn.masked_linear import (
     wrap_kernel_init,
 )
 from netket.utils.types import Array, DType, NNInitFunc
-from netket.utils import deprecate_dtype
 
 
-@deprecate_dtype
 class FastMaskedDense1D(nn.Module):
     """
     1D linear transformation module with mask for fast autoregressive NN.
 
-    See :class:`netket.nn.FastMaskedConv1D` for a brief explanation of fast autoregressive sampling.
+    See :class:`netket.models.FastARNNSequential` for a brief explanation of fast autoregressive sampling.
 
     TODO: FastMaskedDense1D does not support JIT yet, because it involves slicing the cached inputs
     and the weights with a dynamic shape.
@@ -116,8 +114,7 @@ class FastMaskedDense1D(nn.Module):
             "cache", "inputs", zeros, None, (batch, size, in_features), inputs.dtype
         )
 
-        initializing = self.is_mutable_collection("params")
-        if not initializing:
+        if not self.is_initializing():
             # Add the input site into the cache
             # To write the cache, use `_cache.value` as the left value of the assignment
             _cache.value = jnp.where(
@@ -125,6 +122,7 @@ class FastMaskedDense1D(nn.Module):
                 _cache.value.at[:, index - self.exclusive, :].set(inputs),
                 _cache.value,
             )
+
         cache = _cache.value
 
         cache_i = cache[:, :size_i, :]
@@ -141,7 +139,7 @@ class FastMaskedDense1D(nn.Module):
         y_i = lax.dot(cache_i, mask_i * kernel_i, precision=self.precision)
 
         if self.use_bias:
-            y_i = y_i + bias[index, :]
+            y_i = y_i + bias[index, :]  # type: ignore[assignment,index]
 
         assert y_i.shape[1] == self.features
 
@@ -160,18 +158,14 @@ class FastMaskedDense1D(nn.Module):
         Returns:
           The transformed data.
         """
-        return MaskedDense1D.__call__(self, inputs)
+        return MaskedDense1D.__call__(self, inputs)  # type: ignore[arg-type]
 
 
-@deprecate_dtype
 class FastMaskedConv1D(nn.Module):
     """
     1D convolution module with mask for fast autoregressive NN.
 
-    The fast autoregressive sampling is described in `Ramachandran et. {\\it al} <https://arxiv.org/abs/1704.06001>`_.
-    To generate one sample using an autoregressive network, we need to evaluate the network `N` times, where `N` is
-    the number of input sites. But we only change one input site each time, so we can cache unchanged intermediate results
-    and avoid repeated computation.
+    See :class:`netket.models.FastARNNSequential` for a brief explanation of fast autoregressive sampling.
     """
 
     features: int
@@ -247,8 +241,7 @@ class FastMaskedConv1D(nn.Module):
             inputs.dtype,
         )
 
-        initializing = self.is_mutable_collection("params")
-        if not initializing:
+        if not self.is_initializing():
             # Add the input site into the cache
             # To write the cache, use `_cache.value` as the left value of the assignment
             _cache.value = jnp.where(
@@ -267,7 +260,7 @@ class FastMaskedConv1D(nn.Module):
         dimension_numbers = _conv_dimension_numbers(cache.shape)
         y_i = lax.conv_general_dilated(
             cache,
-            kernel,
+            kernel,  # type: ignore[arg-type]
             window_strides=(1,),
             padding="VALID",
             lhs_dilation=(1,),
@@ -300,21 +293,20 @@ class FastMaskedConv1D(nn.Module):
         return MaskedConv1D.__call__(self, inputs)
 
 
-@deprecate_dtype
 class FastMaskedConv2D(nn.Module):
     """
     2D convolution module with mask for fast autoregressive NN.
 
-    See :class:`netket.nn.FastMaskedConv1D` for a brief explanation of fast autoregressive sampling.
+    See :class:`netket.models.FastARNNSequential` for a brief explanation of fast autoregressive sampling.
     """
 
     L: int
     """edge length of the 2D lattice."""
     features: int
     """number of convolution filters."""
-    kernel_size: Tuple[int, int]
+    kernel_size: tuple[int, int]
     """shape of the convolutional kernel `(h, w)`. Typically, :math:`h = w // 2 + 1`."""
-    kernel_dilation: Tuple[int, int]
+    kernel_dilation: tuple[int, int]
     """a sequence of 2 integers, giving the dilation factor to
     apply in each spatial dimension of the convolution kernel."""
     exclusive: bool
@@ -396,8 +388,7 @@ class FastMaskedConv2D(nn.Module):
             inputs.dtype,
         )
 
-        initializing = self.is_mutable_collection("params")
-        if not initializing:
+        if not self.is_initializing():
             # Add the input site into the cache
             # To write the cache, use `_cache.value` as the left value of the assignment
 
