@@ -1,16 +1,32 @@
-import abc
-from typing import Tuple
+# Copyright 2021-2023 The NetKet Authors - All rights reserved.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import numpy as np
+import jax
 import jax.numpy as jnp
 
 from numba import jit
 from scipy.sparse import csr_matrix as _csr_matrix
+from scipy.sparse import issparse
 
+from netket import config
 from netket.hilbert import DiscreteHilbert
 from netket.operator import AbstractOperator
+from netket.utils.optional_deps import import_optional_dependency
 
 
-class DiscreteOperator(AbstractOperator):
+class DiscreteOperator(AbstractOperator[DiscreteHilbert]):
     r"""This class is the base class for operators defined on a
     discrete Hilbert space. Users interested in implementing new
     quantum Operators for discrete Hilbert spaces should derive
@@ -29,7 +45,7 @@ class DiscreteOperator(AbstractOperator):
         """The maximum number of non zero ⟨x|O|x'⟩ for every x."""
         raise NotImplementedError
 
-    def get_conn_padded(self, x: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+    def get_conn_padded(self, x: np.ndarray) -> tuple[np.ndarray, np.ndarray]:
         r"""Finds the connected elements of the Operator.
 
         Starting from a batch of quantum numbers :math:`x={x_1, ... x_n}` of
@@ -44,11 +60,17 @@ class DiscreteOperator(AbstractOperator):
         Args:
             x : A N-tensor of shape :math:`(...,hilbert.size)` containing
                 the batch/batches of quantum numbers :math:`x`.
+
         Returns:
             **(x_primes, mels)**: The connected states x', in a N+1-tensor and an
             N-tensor containing the matrix elements :math:`O(x,x')`
             associated to each x' for every batch.
         """
+        if config.netket_experimental_sharding and jax.device_count() > 1:
+            raise RuntimeError(
+                "When using Sharding mode, only jax operators are supported."
+            )
+
         n_visible = x.shape[-1]
         n_samples = x.size // n_visible
 
@@ -64,10 +86,9 @@ class DiscreteOperator(AbstractOperator):
 
         return x_primes_r, mels_r
 
-    @abc.abstractmethod
     def get_conn_flattened(
-        self, x: np.ndarray, sections: np.ndarray
-    ) -> Tuple[np.ndarray, np.ndarray]:
+        self, x: np.ndarray, sections: np.ndarray, pad: bool = False
+    ) -> tuple[np.ndarray, np.ndarray]:
         r"""Finds the connected elements of the Operator.
 
         Starting from a given quantum number :math:`x`, it finds all
@@ -78,13 +99,17 @@ class DiscreteOperator(AbstractOperator):
         :math:`k=0,1...N_{\mathrm{connected}}`.
 
         This is a batched version, where x is a matrix of shape
-        :math:`(batch_size,hilbert.size)`.
+        :code:`(batch_size,hilbert.size)`.
 
         Args:
-            x (matrix): A matrix of shape (batch_size,hilbert.size)
+            x: A matrix of shape `(batch_size, hilbert.size)`
                 containing the batch of quantum numbers x.
-            sections (array): An array of sections for the flattened x'.
+            sections: An array of sections for the flattened x'.
                 See numpy.split for the meaning of sections.
+            pad: If True, the output x' is padded to the maximum
+                number of connected states. If False, the output x'
+                is not padded and the number of connected states
+                is given by the sections array. (Defaults: ``False``)
 
         Returns:
             (matrix, array): The connected states x', flattened together in
@@ -93,6 +118,19 @@ class DiscreteOperator(AbstractOperator):
                 associated to each x'.
 
         """
+        raise NotImplementedError(
+            f"""
+            The method get_conn_flattened has not been implemented for the object of
+            type {type(self)}.
+
+            This may happen if you defined a custom class inheriting from DiscreteOperator
+            and you have not implemented this method. In that case, you should define
+            `get_conn_flattened(self, x: array, sections: array)` according to the
+            docstring provided on the documentation.
+
+            Otherwise, please open an issue on netket's github repository.
+            """
+        )
 
     def get_conn(self, x: np.ndarray):
         r"""Finds the connected elements of the Operator. Starting
@@ -100,11 +138,14 @@ class DiscreteOperator(AbstractOperator):
         that the matrix element :math:`O(x,x')` is different from zero. In general there
         will be several different connected states x' satisfying this
         condition, and they are denoted here :math:`x'(k)`, for :math:`k=0,1...N_{\mathrm{connected}}`.
+
         Args:
-            x (array): An array of shape (hilbert.size) containing the quantum numbers x.
+            x: An array of shape `(hilbert.size, )` containing the quantum numbers x.
+
         Returns:
             matrix: The connected states x' of shape (N_connected,hilbert.size)
             array: An array containing the matrix elements :math:`O(x,x')` associated to each x'.
+
         Raise:
             ValueError: If the given quantum number is not compatible with the hilbert space.
         """
@@ -114,7 +155,8 @@ class DiscreteOperator(AbstractOperator):
             )
         if x.shape[0] != self.hilbert.size:
             raise ValueError(
-                "The given quantum numbers do not match the hilbert space."
+                "The given quantum numbers do not match the hilbert space because"
+                f"it has shape {x.shape} of which[0] but expected {self.hilbert.size}."
             )
 
         return self.get_conn_flattened(
@@ -135,7 +177,7 @@ class DiscreteOperator(AbstractOperator):
 
         """
         if out is None:
-            out = np.empty(x.shape[0], dtype=np.intc)
+            out = np.empty(x.shape[0], dtype=np.int32)
         self.get_conn_flattened(x, out)
         out = self._n_conn_from_sections(out)
 
@@ -209,9 +251,9 @@ class DiscreteOperator(AbstractOperator):
         Returns:
             A :class:`qutip.Qobj` object.
         """
-        from qutip import Qobj
+        qutip = import_optional_dependency("qutip", descr="to_qobj")
 
-        return Qobj(
+        return qutip.Qobj(
             self.to_sparse(), dims=[list(self.hilbert.shape), list(self.hilbert.shape)]
         )
 
@@ -220,10 +262,14 @@ class DiscreteOperator(AbstractOperator):
 
     def apply(self, v: np.ndarray) -> np.ndarray:
         op = self.to_linear_operator()
-        return op.dot(v)
+        return op @ v
 
     def __matmul__(self, other):
-        if isinstance(other, np.ndarray) or isinstance(other, jnp.ndarray):
+        if (
+            isinstance(other, np.ndarray)
+            or isinstance(other, jnp.ndarray)
+            or issparse(other)
+        ):
             return self.apply(other)
         elif isinstance(other, AbstractOperator):
             return self._op__matmul__(other)
@@ -235,7 +281,11 @@ class DiscreteOperator(AbstractOperator):
         return NotImplemented
 
     def __rmatmul__(self, other):
-        if isinstance(other, np.ndarray) or isinstance(other, jnp.ndarray):
+        if (
+            isinstance(other, np.ndarray)
+            or isinstance(other, jnp.ndarray)
+            or issparse(other)
+        ):
             # return self.apply(other)
             return NotImplemented
         elif isinstance(other, AbstractOperator):
@@ -249,14 +299,3 @@ class DiscreteOperator(AbstractOperator):
 
     def to_linear_operator(self):
         return self.to_sparse()
-
-    def _get_conn_flattened_closure(self):
-        raise NotImplementedError(
-            """
-            _get_conn_flattened_closure not implemented for this operator type.
-            You were probably trying to use an operator with a sampler.
-            Please report this bug.
-
-            numba4jax won't work.
-            """
-        )
