@@ -12,12 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import List as PyList
-import functools
-import warnings
 
 import numpy as np
-import jax.numpy as jnp
 import numba
 from numba import jit
 from numba.typed import List
@@ -25,9 +21,13 @@ from numba.typed import List
 from scipy.sparse.linalg import LinearOperator
 
 import netket.jax as nkjax
+from netket.jax import canonicalize_dtypes
+from netket.utils.optional_deps import import_optional_dependency
+from netket.utils.types import DType
 
 from ._discrete_operator import DiscreteOperator
-from ._local_operator import LocalOperator
+from ._discrete_operator_jax import DiscreteJaxOperator
+from ._local_operator import LocalOperatorNumba as LocalOperator
 from ._abstract_super_operator import AbstractSuperOperator
 
 
@@ -69,30 +69,26 @@ class LocalLiouvillian(AbstractSuperOperator):
     def __init__(
         self,
         ham: DiscreteOperator,
-        jump_ops: PyList[DiscreteOperator] = [],
-        dtype=None,
+        jump_ops: list[DiscreteOperator] = [],
+        dtype: DType | None = None,
     ):
+        if isinstance(ham, DiscreteJaxOperator):
+            ham = ham.to_numba_operator()
         super().__init__(ham.hilbert)
 
-        if dtype is None:
-            dtype = jnp.promote_types(complex, ham.dtype)
-            dtype = functools.reduce(
-                lambda dt, op: jnp.promote_types(dt, op.dtype), jump_ops, dtype
-            )
-        elif not nkjax.is_complex_dtype(dtype):
-            old_dtype = dtype
-            dtype = jnp.promote_types(complex, old_dtype)
-            warnings.warn(
-                np.ComplexWarning(
-                    f"A complex dtype is required (dtype={old_dtype} specified). "
-                    f"Promoting to dtype={dtype}."
-                )
-            )
+        dtype = canonicalize_dtypes(complex, ham, *jump_ops, dtype=dtype)
 
-        dtype = np.empty((), dtype=dtype).dtype
+        if not nkjax.is_complex_dtype(dtype):
+            raise TypeError(f"A complex dtype is required (dtype={dtype} specified).")
+
+        self._jump_ops = []
+        for op in jump_ops:
+            if isinstance(op, DiscreteJaxOperator):
+                op = op.to_numba_operator()
+
+            self._jump_ops.append(op.copy(dtype=dtype))
 
         self._H = ham
-        self._jump_ops = [op.copy(dtype=dtype) for op in jump_ops]  # to accept dicts
         self._Hnh = ham
         self._max_dissipator_conn_size = 0
         self._max_conn_size = 0
@@ -120,7 +116,7 @@ class LocalLiouvillian(AbstractSuperOperator):
         return self._Hnh
 
     @property
-    def jump_operators(self) -> PyList[LocalOperator]:
+    def jump_operators(self) -> list[LocalOperator]:
         """The list of local operators in this Liouvillian"""
         return self._jump_ops
 
@@ -156,11 +152,15 @@ class LocalLiouvillian(AbstractSuperOperator):
         self._mels_f = np.empty(max_conn_size, dtype=self.dtype)
 
     def add_jump_operator(self, op):
+        if isinstance(op, DiscreteJaxOperator):
+            op = op.to_numba_operator()
         self._jump_ops.append(op)
         self._compute_hnh()
 
     def add_jump_operators(self, ops):
         for op in ops:
+            if isinstance(op, DiscreteJaxOperator):
+                op = op.to_numba_operator()
             self._jump_ops.append(op)
 
         self._compute_hnh()
@@ -238,7 +238,7 @@ class LocalLiouvillian(AbstractSuperOperator):
 
         sections_Lr = np.empty(batch_size * n_jops, dtype=np.int32)
         sections_Lc = np.empty(batch_size * n_jops, dtype=np.int32)
-        for (i, L) in enumerate(self._jump_ops):
+        for i, L in enumerate(self._jump_ops):
             L_xrp, L_mel_r = L.get_conn_flattened(
                 xr, sections_Lr[i * batch_size : (i + 1) * batch_size]
             )
@@ -474,8 +474,8 @@ class LocalLiouvillian(AbstractSuperOperator):
         Returns:
             A :class:`qutip.liouvillian` object.
         """
-        from qutip import liouvillian
+        qutip = import_optional_dependency("qutip", descr="to_qobj")
 
-        return liouvillian(
+        return qutip.liouvillian(
             self.hamiltonian.to_qobj(), [op.to_qobj() for op in self.jump_operators]
         )
