@@ -11,59 +11,74 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from typing import Optional, Callable
-from functools import partial
+from collections.abc import Callable
+from collections.abc import Hashable
+
+import jax
+import jax.numpy as jnp
 
 from netket.utils.types import DType, PyTree, Array
 
 from netket.hilbert import AbstractHilbert
 from netket.operator import ContinuousOperator
-
-import jax
-import jax.numpy as jnp
+from netket.utils import HashableArray, struct
 
 
 class PotentialEnergy(ContinuousOperator):
     r"""Returns the local potential energy defined in afun"""
 
+    _potential_fun: Callable = struct.static_field()
+    _coefficient: Array
+
     def __init__(
         self,
         hilbert: AbstractHilbert,
         afun: Callable,
         coefficient: float = 1.0,
-        dtype: Optional[DType] = float,
+        dtype: DType | None = None,
     ):
         r"""
         Args:
             hilbert: The underlying Hilbert space on which the operator is defined
             afun: The potential energy as function of x
-            coefficients: A coefficient for the ContinuousOperator object
-            dtype: Data type of the matrix elements. Defaults to `np.float64`
+            coefficient: A coefficient for the ContinuousOperator object
+            dtype: Data type of the coefficient
         """
 
-        self._afun = afun
+        self._potential_fun = afun
+        self._coefficient = jnp.array(coefficient, dtype=dtype)
 
-        self.coefficient = jnp.array(coefficient, dtype=dtype)
-
-        super().__init__(hilbert, self.coefficient.dtype)
-
-    def _expect_kernel_single(
-        self, logpsi: Callable, params: PyTree, x: Array, coefficient: Optional[PyTree]
-    ):
-        return coefficient * self._afun(x)
-
-    @partial(jax.vmap, in_axes=(None, None, None, 0, None))
-    def _expect_kernel(
-        self, logpsi: Callable, params: PyTree, x: Array, coefficient: Optional[PyTree]
-    ):
-        return self._expect_kernel_single(logpsi, params, x, coefficient)
+        super().__init__(hilbert, self._coefficient.dtype)
 
     @property
-    def is_hermitian(self):
+    def coefficient(self) -> Array:
+        return self._coefficient
+
+    @property
+    def is_hermitian(self) -> bool:
         return True
 
-    def _pack_arguments(self):
-        return self.coefficient
+    def _expect_kernel(
+        self,
+        logpsi: Callable,
+        params: PyTree,
+        x: Array,
+    ) -> Array:
+        x_dtype = jnp.promote_types(self.dtype, x.dtype)
+        return self.coefficient * jax.vmap(self._potential_fun, in_axes=(0,))(
+            x.astype(x_dtype)
+        )
+
+    @struct.property_cached(pytree_ignore=True)
+    def _attrs(self) -> tuple[Hashable, ...]:
+        return (
+            self.hilbert,
+            self._potential_fun,
+            self.dtype,
+            HashableArray(self.coefficient),
+        )
 
     def __repr__(self):
-        return f"Potential(coefficient={self.coefficient}, function={self._afun})"
+        return (
+            f"Potential(coefficient={self.coefficient}, function={self._potential_fun})"
+        )
