@@ -12,18 +12,46 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TypeVar as _TypeVar
 from scipy import sparse as _sparse
-
 from netket.utils.types import DType as _DType
 
-from netket.hilbert import AbstractHilbert as _AbstractHilbert
+from netket.hilbert import DiscreteHilbert as _DiscreteHilbert
 
-from ._local_operator import LocalOperator as _LocalOperator
+from ._local_operator import (
+    LocalOperator as _LocalOperator,
+    LocalOperatorBase as _LocalOperatorBase,
+)
+
+_LocalOperatorT = _TypeVar("_LocalOperatorT", bound=_LocalOperatorBase)
+
+
+def identity(
+    hilbert: _DiscreteHilbert,
+    dtype: _DType = None,
+    *,
+    cls: type[_LocalOperatorT] = _LocalOperator,
+) -> _LocalOperatorT:
+    """
+    Builds the :math:`\\mathbb{I}` identity operator.
+
+    Args:
+        hilbert: The hilbert space.
+        dtype: The datatype to use for the matrix elements.
+
+    Returns:
+        An instance of {class}`nk.operator.LocalOperator`.
+    """
+    return cls(hilbert, constant=1.0, dtype=dtype)
 
 
 def sigmax(
-    hilbert: _AbstractHilbert, site: int, dtype: _DType = float
-) -> _LocalOperator:
+    hilbert: _DiscreteHilbert,
+    site: int,
+    dtype: _DType = None,
+    *,
+    cls: type[_LocalOperatorT] = _LocalOperator,
+) -> _LocalOperatorT:
     """
     Builds the :math:`\\sigma^x` operator acting on the `site`-th of the Hilbert
     space `hilbert`.
@@ -31,9 +59,13 @@ def sigmax(
     If `hilbert` is a non-Spin space of local dimension M, it is considered
     as a (M-1)/2 - spin space.
 
-    :param hilbert: The hilbert space
-    :param site: the site on which this operator acts
-    :return: a nk.operator.LocalOperator
+    Args:
+        hilbert: The hilbert space.
+        site: The site on which this operator acts.
+        dtype: The datatype to use for the matrix elements.
+
+    Returns:
+        An instance of {class}`nk.operator.LocalOperator`.
     """
     import numpy as np
 
@@ -43,12 +75,16 @@ def sigmax(
     D = [np.sqrt((S + 1) * 2 * a - a * (a + 1)) for a in np.arange(1, N)]
     mat = np.diag(D, 1) + np.diag(D, -1)
     mat = _sparse.coo_matrix(mat)
-    return _LocalOperator(hilbert, mat, [site], dtype=dtype)
+    return cls(hilbert, mat, [site], dtype=dtype)
 
 
 def sigmay(
-    hilbert: _AbstractHilbert, site: int, dtype: _DType = complex
-) -> _LocalOperator:
+    hilbert: _DiscreteHilbert,
+    site: int,
+    dtype: _DType = None,
+    *,
+    cls: type[_LocalOperatorT] = _LocalOperator,
+) -> _LocalOperatorT:
     """
     Builds the :math:`\\sigma^y` operator acting on the `site`-th of the Hilbert
     space `hilbert`.
@@ -56,12 +92,22 @@ def sigmay(
     If `hilbert` is a non-Spin space of local dimension M, it is considered
     as a (M-1)/2 - spin space.
 
-    :param hilbert: The hilbert space
-    :param site: the site on which this operator acts
-    :return: a nk.operator.LocalOperator
+    Args:
+        hilbert: The hilbert space.
+        site: The site on which this operator acts.
+        dtype: The datatype to use for the matrix elements.
+
+    Returns:
+        An instance of {class}`nk.operator.LocalOperator`.
     """
     import numpy as np
     import netket.jax as nkjax
+    from netket.utils import module_version
+
+    if module_version(np) >= (2, 0, 0):
+        from numpy.exceptions import ComplexWarning
+    else:
+        from numpy import ComplexWarning  # type: ignore
 
     if not nkjax.is_complex_dtype(dtype):
         import jax.numpy as jnp
@@ -69,12 +115,14 @@ def sigmay(
 
         old_dtype = dtype
         dtype = jnp.promote_types(complex, old_dtype)
-        warnings.warn(
-            np.ComplexWarning(
-                f"A complex dtype is required (dtype={old_dtype} specified). "
-                f"Promoting to dtype={dtype}."
+        if old_dtype is not None:
+            warnings.warn(
+                ComplexWarning(
+                    f"A complex dtype is required (dtype={old_dtype} specified). "
+                    f"Promoting to dtype={dtype}."
+                ),
+                stacklevel=2,
             )
-        )
 
     N = hilbert.size_at_index(site)
     S = (N - 1) / 2
@@ -82,12 +130,16 @@ def sigmay(
     D = np.array([1j * np.sqrt((S + 1) * 2 * a - a * (a + 1)) for a in np.arange(1, N)])
     mat = np.diag(D, -1) + np.diag(-D, 1)
     mat = _sparse.coo_matrix(mat)
-    return _LocalOperator(hilbert, mat, [site], dtype=dtype)
+    return cls(hilbert, mat, [site], dtype=dtype)
 
 
 def sigmaz(
-    hilbert: _AbstractHilbert, site: int, dtype: _DType = float
-) -> _LocalOperator:
+    hilbert: _DiscreteHilbert,
+    site: int,
+    dtype: _DType = None,
+    *,
+    cls: type[_LocalOperatorT] = _LocalOperator,
+) -> _LocalOperatorT:
     """
     Builds the :math:`\\sigma^z` operator acting on the `site`-th of the Hilbert
     space `hilbert`.
@@ -95,9 +147,13 @@ def sigmaz(
     If `hilbert` is a non-Spin space of local dimension M, it is considered
     as a (M-1)/2 - spin space.
 
-    :param hilbert: The hilbert space
-    :param site: the site on which this operator acts
-    :return: a nk.operator.LocalOperator
+    Args:
+        hilbert: The hilbert space.
+        site: The site on which this operator acts.
+        dtype: The datatype to use for the matrix elements.
+
+    Returns:
+        An instance of {class}`nk.operator.LocalOperator`.
     """
     import numpy as np
 
@@ -107,12 +163,16 @@ def sigmaz(
     D = np.array([2 * m for m in np.arange(S, -(S + 1), -1)])
     mat = np.diag(D, 0)
     mat = _sparse.coo_matrix(mat)
-    return _LocalOperator(hilbert, mat, [site], dtype=dtype)
+    return cls(hilbert, mat, [site], dtype=dtype)
 
 
 def sigmam(
-    hilbert: _AbstractHilbert, site: int, dtype: _DType = float
-) -> _LocalOperator:
+    hilbert: _DiscreteHilbert,
+    site: int,
+    dtype: _DType = None,
+    *,
+    cls: type[_LocalOperatorT] = _LocalOperator,
+) -> _LocalOperatorT:
     """
     Builds the :math:`\\sigma^{-} = \\frac{1}{2}(\\sigma^x - i \\sigma^y)` operator acting on the
     `site`-th of the Hilbert space `hilbert`.
@@ -120,9 +180,13 @@ def sigmam(
     If `hilbert` is a non-Spin space of local dimension M, it is considered
     as a (M-1)/2 - spin space.
 
-    :param hilbert: The hilbert space
-    :param site: the site on which this operator acts
-    :return: a nk.operator.LocalOperator
+    Args:
+        hilbert: The hilbert space.
+        site: The site on which this operator acts.
+        dtype: The datatype to use for the matrix elements.
+
+    Returns:
+        An instance of {class}`nk.operator.LocalOperator`.
     """
     import numpy as np
 
@@ -133,12 +197,16 @@ def sigmam(
     D = np.array([np.sqrt(S2 - m * (m - 1)) for m in np.arange(S, -S, -1)])
     mat = np.diag(D, -1)
     mat = _sparse.coo_matrix(mat)
-    return _LocalOperator(hilbert, mat, [site], dtype=dtype)
+    return cls(hilbert, mat, [site], dtype=dtype)
 
 
 def sigmap(
-    hilbert: _AbstractHilbert, site: int, dtype: _DType = float
-) -> _LocalOperator:
+    hilbert: _DiscreteHilbert,
+    site: int,
+    dtype: _DType = None,
+    *,
+    cls: type[_LocalOperatorT] = _LocalOperator,
+) -> _LocalOperatorT:
     """
     Builds the :math:`\\sigma^{+} = \\frac{1}{2}(\\sigma^x + i \\sigma^y)` operator acting on the
     `site`-th of the Hilbert space `hilbert`.
@@ -146,9 +214,13 @@ def sigmap(
     If `hilbert` is a non-Spin space of local dimension M, it is considered
     as a (M-1)/2 - spin space.
 
-    :param hilbert: The hilbert space
-    :param site: the site on which this operator acts
-    :return: a nk.operator.LocalOperator
+    Args:
+        hilbert: The hilbert space.
+        site: The site on which this operator acts.
+        dtype: The datatype to use for the matrix elements.
+
+    Returns:
+        An instance of {class}`nk.operator.LocalOperator`.
     """
     import numpy as np
 
@@ -159,4 +231,4 @@ def sigmap(
     D = np.array([np.sqrt(S2 - m * (m + 1)) for m in np.arange(S - 1, -(S + 1), -1)])
     mat = np.diag(D, 1)
     mat = _sparse.coo_matrix(mat)
-    return _LocalOperator(hilbert, mat, [site], dtype=dtype)
+    return cls(hilbert, mat, [site], dtype=dtype)
