@@ -12,12 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Callable, Optional, Tuple, Any
+from typing import Any
+from collections.abc import Callable
 
 import jax
 from jax import numpy as jnp
 from flax import struct
 
+from netket.utils import timing
 from netket.utils.types import PyTree
 
 
@@ -27,7 +29,7 @@ class _Uninitialized:
 
 Uninitialized = _Uninitialized()
 
-SolverT = Callable[["LinearOperator", PyTree], Tuple[PyTree, Any]]
+SolverT = Callable[["LinearOperator", PyTree], tuple[PyTree, Any]]
 """Signature for the solver used by a linear operator."""
 
 
@@ -71,7 +73,7 @@ class LinearOperator:
     # PUBLIC API: METHOD TO EXTEND Optionally IF YOU WANT TO DEFINE A NEW S object with
     # custom logic
     def _solve(
-        self, solve_fun: SolverT, y: PyTree, *, x0: Optional[PyTree] = None, **kwargs
+        self, solve_fun: SolverT, y: PyTree, *, x0: PyTree | None = None, **kwargs
     ) -> PyTree:
         # dont pass x0 if it's unset.
         # some solvers might not need/require it.
@@ -81,8 +83,9 @@ class LinearOperator:
         return solve_fun(self, y, **kwargs)
 
     # PUBLIC API: Extend _solve
+    @timing.timed
     def solve(
-        self, solve_fun: SolverT, y: PyTree, *, x0: Optional[PyTree] = None, **kwargs
+        self, solve_fun: SolverT, y: PyTree, *, x0: PyTree | None = None, **kwargs
     ) -> PyTree:
         """
         Solve the linear system x=⟨S⟩⁻¹⟨y⟩ with the chosen iterative solver.
@@ -100,6 +103,7 @@ class LinearOperator:
         return self._solve(jax.tree_util.Partial(solve_fun), y, x0=x0, **kwargs)
 
     # PUBLIC API: METHOD TO EXTEND IF YOU WANT TO DEFINE A NEW S object
+    @timing.timed
     def to_dense(self) -> jnp.ndarray:
         """
         Convert the lazy matrix representation to a dense matrix representation.s
