@@ -12,25 +12,33 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Callable, Optional, Union
-from functools import partial
+from collections.abc import Callable
+import warnings
 
 import jax
 from jax import numpy as jnp
 from flax import struct
 
 import netket.jax as nkjax
+from netket.utils import timing, HashablePartial
 from netket.utils.types import PyTree
-
-from netket.nn import split_array_mpi
+from netket.utils.api_utils import partial_from_kwargs
+from netket.errors import (
+    IllegalHolomorphicDeclarationForRealParametersError,
+    NonHolomorphicQGTOnTheFlyDenseRepresentationError,
+    HolomorphicUndeclaredWarning,
+)
 
 from .common import check_valid_vector_type
 from .qgt_onthefly_logic import mat_vec_factory, mat_vec_chunked_factory
 
-from ..linear_operator import LinearOperator, Uninitialized
+from ..linear_operator import LinearOperator, SolverT, Uninitialized
 
 
-def QGTOnTheFly(vstate=None, *, chunk_size=None, **kwargs) -> "QGTOnTheFlyT":
+@partial_from_kwargs
+def QGTOnTheFly(
+    vstate, *, chunk_size=None, holomorphic: bool | None = None, **kwargs
+) -> "QGTOnTheFlyT":
     """
     Lazy representation of an S Matrix computed by performing 2 jvp
     and 1 vjp products, using the variational state's model, the
@@ -47,9 +55,6 @@ def QGTOnTheFly(vstate=None, *, chunk_size=None, **kwargs) -> "QGTOnTheFlyT":
                     (useful for models where the backward pass requires more
                     memory than the forward pass).
     """
-    if vstate is None:
-        return partial(QGTOnTheFly, chunk_size=chunk_size, **kwargs)
-
     if kwargs.pop("diag_scale", None) is not None:
         raise NotImplementedError(
             "\n`diag_scale` argument is not yet supported by QGTOnTheFly."
@@ -59,44 +64,100 @@ def QGTOnTheFly(vstate=None, *, chunk_size=None, **kwargs) -> "QGTOnTheFlyT":
         )
 
     # TODO: Find a better way to handle this case
-    from netket.vqs import ExactState
+    from netket.vqs import FullSumState
 
-    if isinstance(vstate, ExactState):
-        samples = split_array_mpi(vstate._all_states)
-        pdf = split_array_mpi(vstate.probability_distribution())
+    if isinstance(vstate, FullSumState):
+        samples = vstate._all_states
+        pdf = vstate.probability_distribution()
     else:
-        if jnp.ndim(vstate.samples) == 2:
-            samples = vstate.samples
-        else:
-            samples = vstate.samples.reshape((-1, vstate.samples.shape[-1]))
+        samples = vstate.samples
         pdf = None
 
-    if chunk_size is None and hasattr(vstate, "chunk_size"):
-        chunk_size = vstate.chunk_size
+    if chunk_size is None:
+        chunk_size = getattr(vstate, "chunk_size", None)
 
-    n_samples = samples.shape[0]
+    return QGTOnTheFly_DefaultConstructor(
+        vstate._apply_fun,
+        vstate.parameters,
+        vstate.model_state,
+        samples,
+        pdf=pdf,
+        chunk_size=chunk_size,
+        holomorphic=holomorphic,
+        **kwargs,
+    )
 
-    if chunk_size is None or chunk_size >= n_samples:
+
+@timing.timed
+def QGTOnTheFly_DefaultConstructor(
+    apply_fun,
+    parameters,
+    model_state,
+    samples,
+    pdf=None,
+    *,
+    chunk_size: int | None = None,
+    holomorphic: bool | None = None,
+    **kwargs,
+) -> "QGTOnTheFlyT":
+    """ """
+    if pdf is not None:
+        if not pdf.shape == samples.shape[:-1]:
+            raise ValueError(
+                "The shape of pdf must match the shape of the samples, "
+                f"instead you provided (pdf.shape={pdf.shape}) != "
+                f"(samples.shape={samples.shape[:-1]})"
+            )
+        if pdf.ndim >= 2:
+            pdf = jax.jit(jax.lax.collapse, static_argnums=(1, 2))(pdf, 0, 2)
+
+    # The code does not support an extra batch dimension
+    if samples.ndim >= 3:
+        # use jit so that we can do it on global shared array
+        samples = jax.jit(jax.lax.collapse, static_argnums=(1, 2))(samples, 0, 2)
+
+    n_samples_per_rank = samples.shape[0] // jax.device_count()
+    if chunk_size is None or chunk_size >= n_samples_per_rank:
         mv_factory = mat_vec_factory
         chunking = False
     else:
-        samples, _ = nkjax.chunk(samples, chunk_size)
-        if pdf is not None:
-            pdf, _ = nkjax.chunk(pdf, chunk_size)
-        mv_factory = mat_vec_chunked_factory
+        mv_factory = HashablePartial(mat_vec_chunked_factory, chunk_size=chunk_size)
         chunking = True
 
+    # check if holomorphic or not
+    if holomorphic:
+        if nkjax.tree_leaf_isreal(parameters):
+            raise IllegalHolomorphicDeclarationForRealParametersError()
+        else:
+            mode = "holomorphic"
+    else:
+        if not nkjax.tree_leaf_iscomplex(parameters):
+            mode = "real"
+        else:
+            if holomorphic is None:
+                warnings.warn(HolomorphicUndeclaredWarning(), UserWarning)
+            mode = "complex"
+
+    nkjax.jacobian_default_mode(
+        apply_fun,
+        parameters,
+        model_state,
+        samples,
+        holomorphic=holomorphic,
+    )
+
     mat_vec = mv_factory(
-        forward_fn=vstate._apply_fun,
-        params=vstate.parameters,
-        model_state=vstate.model_state,
+        forward_fn=apply_fun,
+        params=parameters,
+        model_state=model_state,
         samples=samples,
         pdf=pdf,
     )
     return QGTOnTheFlyT(
         _mat_vec=mat_vec,
-        _params=vstate.parameters,
+        _params=parameters,
         _chunking=chunking,
+        _mode=mode,
         **kwargs,
     )
 
@@ -114,21 +175,33 @@ class QGTOnTheFlyT(LinearOperator):
     the field `sr`.
     """
 
-    _mat_vec: Callable[[PyTree, float], PyTree] = Uninitialized
+    _mat_vec: Callable[[PyTree, float], PyTree] = Uninitialized  # type: ignore
     """The S matrix-vector product as generated by mat_vec_factory.
     It's a jax.Partial, so can be used as pytree_node."""
 
-    _params: PyTree = Uninitialized
+    _params: PyTree = Uninitialized  # type: ignore
     """The first input to apply_fun (parameters of the ansatz).
     Only used as a shape placeholder."""
 
     _chunking: bool = struct.field(pytree_node=False, default=False)
     """Whether the implementation with chunks is used which currently does not support vmapping over it"""
 
+    _mode: str = struct.field(pytree_node=False, default=None)
+    """Differentiation mode:
+        - "real": for real-valued R->R and C->R Ansätze, splits the complex inputs
+                  into real and imaginary part.
+        - "complex": for complex-valued R->C and C->C Ansätze, splits the complex
+                  inputs and outputs into real and imaginary part
+        - "holomorphic": for any Ansätze. Does not split complex values.
+        - "auto": autoselect real or complex.
+    """
+
     def __matmul__(self, y):
         return onthefly_mat_treevec(self, y)
 
-    def _solve(self, solve_fun, y: PyTree, *, x0: Optional[PyTree], **kwargs) -> PyTree:
+    def _solve(
+        self, solve_fun: SolverT, y: PyTree, *, x0: PyTree | None, **kwargs
+    ) -> PyTree:
         return _solve(self, solve_fun, y, x0=x0)
 
     def to_dense(self) -> jnp.ndarray:
@@ -138,6 +211,16 @@ class QGTOnTheFlyT(LinearOperator):
         Returns:
             A dense matrix representation of this S matrix.
         """
+        # This condition will be true if the user specified `holomorphic=False` and
+        # if the parameters are complex. If the parameters are real and the user
+        # did not specify holomorphic we will have mode==real and if holomorphic is
+        # True mode==holomorphic.
+        #
+        # We must check this because the AD implementation will compute the wrong
+        # QGT in that case
+        if self._mode == "complex":
+            raise NonHolomorphicQGTOnTheFlyDenseRepresentationError()
+
         return _to_dense(self)
 
     def __repr__(self):
@@ -146,8 +229,8 @@ class QGTOnTheFlyT(LinearOperator):
 
 @jax.jit
 def onthefly_mat_treevec(
-    S: QGTOnTheFly, vec: Union[PyTree, jnp.ndarray]
-) -> Union[PyTree, jnp.ndarray]:
+    S: QGTOnTheFly, vec: PyTree | jnp.ndarray
+) -> PyTree | jnp.ndarray:
     """
     Perform the lazy mat-vec product, where vec is either a tree with the same structure as
     params or a ravelled vector
@@ -185,16 +268,15 @@ def onthefly_mat_treevec(
 
 @jax.jit
 def _solve(
-    self: QGTOnTheFlyT, solve_fun, y: PyTree, *, x0: Optional[PyTree], **kwargs
+    self: QGTOnTheFlyT, solve_fun, y: PyTree, *, x0: PyTree | None, **kwargs
 ) -> PyTree:
-
     check_valid_vector_type(self._params, y)
 
     y = nkjax.tree_cast(y, self._params)
 
     # we could cache this...
     if x0 is None:
-        x0 = jax.tree_map(jnp.zeros_like, y)
+        x0 = jax.tree_util.tree_map(jnp.zeros_like, y)
 
     out, info = solve_fun(self, y, x0=x0)
     return out, info
