@@ -16,14 +16,14 @@ import jax
 from jax.tree_util import Partial
 from functools import partial
 from netket.stats import subtract_mean
-from netket.utils import mpi
 from netket.jax import tree_conj, tree_axpy
 from netket.jax import (
     scanmap,
     scan_reduce,
     scan_append,
-    unchunk,
+    chunk,
 )
+from netket.jax.sharding import sharding_decorator
 
 # Stochastic Reconfiguration with jvp and vjp
 
@@ -45,15 +45,13 @@ def _mat_vec(jvp_fn, v, diag_shift, pdf=None):
 
     w = jvp_fn(v)
     if pdf is None:
-        w = w * (1.0 / (w.size * mpi.n_nodes))
-        w = subtract_mean(w)  # w/ MPI
+        w = w * (1.0 / w.size)
+        w = subtract_mean(w)  # w/ JAX sharding
     else:
-        w = pdf * (w - mpi.mpi_sum_jax(pdf @ w)[0])
+        w = pdf * (w - pdf @ w)
     # Oᴴw = (wᴴO)ᴴ = (w* O)* since 1D arrays are not transposed
     # vjp_fn packages output into a length-1 tuple
     (res,) = tree_conj(vjp_fn(w.conjugate()))
-    res = jax.tree_map(lambda x: mpi.mpi_sum_jax(x)[0], res)
-
     return tree_axpy(diag_shift, v, res)  # res + diag_shift * v
 
 
@@ -92,67 +90,87 @@ def mat_vec_factory(forward_fn, params, model_state, samples, pdf=None):
 # Methods below are needed for the chunked version of QGTOnTheFly
 
 
-@partial(scanmap, scan_fun=scan_append, argnums=2)
-def _O_jvp(forward_fn, params, samples, v):
-    # TODO apply the transpose of sum_inplace (allreduce) to the arg v here
-    # in order to get correct transposition with MPI
-    _, res = jax.jvp(lambda p: forward_fn(p, samples), (params,), (v,))
+@partial(sharding_decorator, sharded_args_tree=(False, False, True, False, False))
+def _O_jvp(forward_fn, params, samples, v, chunk_size):
+    @partial(scanmap, scan_fun=scan_append, argnums=2)
+    def __O_jvp(forward_fn, params, samples, v):
+        # TODO apply the transpose of sum_inplace (allreduce) to the arg v here
+        # in order to get correct transposition with JAX sharding
+        _, res = jax.jvp(lambda p: forward_fn(p, samples), (params,), (v,))
+        return res
+
+    samples, unchunk_fn = chunk(samples, chunk_size)
+    res = __O_jvp(forward_fn, params, samples, v)
+    return unchunk_fn(res)
+
+
+@partial(
+    sharding_decorator,
+    sharded_args_tree=(False, False, True, True, False),
+    reduction_op_tree=jax.lax.psum,
+)
+def _O_vjp(forward_fn, params, samples, w, chunk_size):
+    @partial(scanmap, scan_fun=scan_reduce, argnums=(2, 3))
+    def __O_vjp(forward_fn, params, samples, w):
+        _, vjp_fun = jax.vjp(forward_fn, params, samples)
+        res, _ = vjp_fun(w)
+        return res
+
+    samples, _ = chunk(samples, chunk_size)
+    w, _ = chunk(w, chunk_size)
+    res = __O_vjp(forward_fn, params, samples, w)
     return res
 
 
-@partial(scanmap, scan_fun=scan_reduce, argnums=(2, 3))
-def _O_vjp(forward_fn, params, samples, w):
-    _, vjp_fun = jax.vjp(forward_fn, params, samples)
-    res, _ = vjp_fun(w)
-    return res
+def _OH_w(forward_fn, params, samples, w, chunk_size):
+    return tree_conj(_O_vjp(forward_fn, params, samples, w.conjugate(), chunk_size))
 
 
-def _OH_w(forward_fn, params, samples, w):
-    return tree_conj(_O_vjp(forward_fn, params, samples, w.conjugate()))
-
-
-def _Odagger_DeltaO_v(forward_fn, params, samples, v, pdf=None):
-    w = _O_jvp(forward_fn, params, samples, v)
+def _Odagger_DeltaO_v(forward_fn, params, samples, v, chunk_size, pdf=None):
+    w = _O_jvp(forward_fn, params, samples, v, chunk_size)
     if pdf is None:
-        w = w * (1.0 / (samples.shape[0] * samples.shape[1] * mpi.n_nodes))
-        w_, chunk_fn = unchunk(w)
-        w = chunk_fn(subtract_mean(w_))  # w/ MPI
+        w = w * (1.0 / samples.shape[0])
+        w = subtract_mean(w)  # w/ JAX sharding
     else:
-        w_, chunk_fn = unchunk(w)
-        # here we assume pdf is chunked,
-        # which we undo as the flat vector is needed here
-        pdf_, _ = unchunk(pdf)
-        w_ = pdf_ * (w_ - mpi.mpi_sum_jax(pdf_ @ w_)[0])
-        w = chunk_fn(w_)
-    res = _OH_w(forward_fn, params, samples, w)
-    return jax.tree_map(lambda x: mpi.mpi_sum_jax(x)[0], res)  # MPI
+        w = pdf * (w - pdf @ w)
+    res = _OH_w(forward_fn, params, samples, w, chunk_size)
+    return res
 
 
 # @partial(jax.jit, static_argnums=1)
-def _mat_vec_chunked(forward_fn, params, samples, v, diag_shift, pdf=None):
-    res = _Odagger_DeltaO_v(forward_fn, params, samples, v, pdf)
+def _mat_vec_chunked(forward_fn, params, samples, v, diag_shift, chunk_size, pdf=None):
+    assert samples.ndim == 2  # require flat samples, axis 0 can be sharded
+    res = _Odagger_DeltaO_v(forward_fn, params, samples, v, chunk_size, pdf)
     return tree_axpy(diag_shift, v, res)
 
 
-def _mat_vec_chunked_transposable(forward_fn, params, samples, v, diag_shift, pdf=None):
+def _mat_vec_chunked_transposable(
+    forward_fn, params, samples, v, diag_shift, chunk_size, pdf=None
+):
     extra_args = (params, samples, diag_shift, pdf)
 
     def _mv(extra_args, x):
         params, samples, diag_shift, pdf = extra_args
-        return _mat_vec_chunked(forward_fn, params, samples, x, diag_shift, pdf)
+        return _mat_vec_chunked(
+            forward_fn, params, samples, x, diag_shift, chunk_size, pdf
+        )
 
     def _mv_trans(extra_args, y):
         # the linear operator is hermitian
         params, samples, diag_shift, pdf = extra_args
         return tree_conj(
-            _mat_vec_chunked(forward_fn, params, samples, tree_conj(y), diag_shift, pdf)
+            _mat_vec_chunked(
+                forward_fn, params, samples, tree_conj(y), diag_shift, chunk_size, pdf
+            )
         )
 
     return jax.custom_derivatives.linear_call(_mv, _mv_trans, extra_args, v)
 
 
-@partial(jax.jit, static_argnums=0)
-def mat_vec_chunked_factory(forward_fn, params, model_state, samples, pdf=None):
+@partial(jax.jit, static_argnums=(0, 5))
+def mat_vec_chunked_factory(
+    forward_fn, params, model_state, samples, pdf=None, chunk_size=None
+):
     """
 
     Prepare a function which computes the regularized SR matrix-vector product
@@ -184,6 +202,9 @@ def mat_vec_chunked_factory(forward_fn, params, model_state, samples, pdf=None):
         return forward_fn({"params": W, **model_state}, samples)
 
     return Partial(
-        partial(_mat_vec_chunked_transposable, fun), params, samples, pdf=pdf
+        partial(_mat_vec_chunked_transposable, fun, chunk_size=chunk_size),
+        params,
+        samples,
+        pdf=pdf,
     )
     # return Partial(lambda f, *args: jax.jit(f)(*args), Partial(partial(_mat_vec_chunked, fun), params, samples))
