@@ -14,11 +14,15 @@
 
 from .base import MetropolisRule
 
+from .fixed import FixedRule
 from .local import LocalRule
 from .exchange import ExchangeRule
 from .hamiltonian import HamiltonianRule
 from .continuous_gaussian import GaussianRule
 from .langevin import LangevinRule
+from .tensor import TensorRule
+from .multiple import MultipleRules
+from .fermion_2nd import FermionHopRule
 
 # numpy backend
 from .local_numpy import LocalRuleNumpy
