@@ -19,7 +19,6 @@
 import itertools
 from abc import ABC
 from dataclasses import dataclass
-from typing import List
 
 import numpy as np
 
@@ -132,7 +131,14 @@ def product(ab: Composite, cd: Composite):  # noqa: F811
 
 @dataclass(frozen=True)
 class FiniteSemiGroup:
-    elems: List[Element]
+    """
+    Base class for Finite Semigroups, made by combining a finite sequence
+    of :class:`netket.utils.group.Element`s.
+
+    """
+
+    elems: list[Element]
+    """List of group elements."""
 
     def __post_init__(self):
         # manually assign self.__hash == ... for frozen dataclass,
