@@ -12,12 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Callable
+from collections.abc import Callable
+
+from flax.linen import Module
 
 import jax.numpy as jnp
 
-from .partial import HashablePartial
-from . import struct
+
+from netket.utils.partial import HashablePartial
+from netket.utils.types import ModuleOrApplyFun, PyTree, Array
+
+from netket.utils import struct
 
 
 def get_afun_if_module(mod_or_fun) -> Callable:
@@ -32,21 +37,21 @@ def get_afun_if_module(mod_or_fun) -> Callable:
 class WrappedApplyFun:
     """Wraps a callable to be a module-like object with the method `apply`."""
 
-    apply: Callable
+    apply: Callable[[PyTree, Array], Array]
     """The wrapped callable."""
 
     def __repr__(self):
         return f"{type(self).__name__}(apply={self.apply}, hash={hash(self)})"
 
 
-def wrap_afun(mod_or_fun):
+def wrap_afun(mod_or_fun: ModuleOrApplyFun) -> Module:
     """Wraps a callable to be a module-like object with the method `apply`.
     Does nothing if it already has an apply method.
     """
     if hasattr(mod_or_fun, "apply"):
-        return mod_or_fun
+        return mod_or_fun  # type: ignore
     else:
-        return WrappedApplyFun(mod_or_fun)
+        return WrappedApplyFun(mod_or_fun)  # type: ignore[return-type]
 
 
 def wrap_to_support_scalar(fun):
