@@ -14,12 +14,11 @@
 
 import sys
 
-from .base import ModuleFramework, framework
+from netket.utils.model_frameworks.base import ModuleFramework, framework
 
 
 @framework
 class FlaxFramework(ModuleFramework):
-
     name: str = "Flax"
 
     @staticmethod
@@ -37,12 +36,4 @@ class FlaxFramework(ModuleFramework):
 
     @staticmethod
     def wrap(module):
-        return module
-
-    @staticmethod
-    def wrap_params(variables):
-        return variables
-
-    @staticmethod
-    def unwrap_params(wrapped_variables):
-        return wrapped_variables
+        return None, module
