@@ -29,25 +29,45 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Utilities for defining custom classes that can be used with jax transformations.
 """
+Utilities for defining custom classes that can be used with jax transformations.
+"""
+from typing import TypeVar, overload
+from collections.abc import Callable
 
+# TODO: switch to typing when we require python 3.11
+from typing_extensions import (
+    dataclass_transform,  # pytype: disable=not-supported-yet
+)
 from functools import partial
 
+
 import dataclasses
+import warnings
 from dataclasses import MISSING
 
 from flax import serialization
 
 import jax
 
-from .utils import _set_new_attribute, _create_fn, get_class_globals
+from netket.utils.struct.utils import (
+    _set_new_attribute,
+    _create_fn,
+    get_class_globals,
+    maximum_positional_args,
+    keyword_arg_names,
+)
+from netket.utils.struct.fields import _cache_name, Uninitialized, field, CachedProperty
+from netket.utils.struct.pytree import Pytree, DATACLASS_USER_INIT_N_ARGS
 
 try:
     from dataclasses import _FIELDS
 except ImportError:
     _FIELDS = "__dataclass_fields__"
 
+_T = TypeVar("_T", bound=type)
+
+
 _CACHES = "__dataclass_caches__"
 
 PRECOMPUTE_CACHED_PROPERTY_NAME = "_precompute_cached_properties"
@@ -58,10 +78,6 @@ _PRE_INIT_NAME = "__pre_init__"
 _DATACLASS_INIT_NAME = "__init_dataclass__"
 
 
-def _cache_name(property_name):
-    return "__" + property_name + "_cache"
-
-
 def _hash_cache_name(class_name):
     return "__" + class_name + "_hash_cache"
 
@@ -70,79 +86,6 @@ def _compute_cache_name(property_name):
     return "__" + property_name
 
 
-## Our stuff
-class _Uninitialized:
-    """
-    Sentinel value used to signal uninitialized values
-    """
-
-    def __repr__(self):
-        return "Uninitialized"
-
-
-Uninitialized = _Uninitialized()
-
-jax.tree_util.register_pytree_node(
-    _Uninitialized, lambda x: ((), Uninitialized), lambda *args: Uninitialized
-)
-
-
-def field(pytree_node=True, serialize=True, cache=False, **kwargs):
-    """Mark a field of a dataclass to be:
-
-    Args:
-        pytree_node: a leaf node in the pytree representation of this dataclass.
-            If False this must be hashable
-        serialize: If True the node is included in the serialization.
-            In general you should not specify this.
-        cache: If True this node is a cache and will be reset every time
-            fields are modified.
-    """
-    return dataclasses.field(
-        metadata={"pytree_node": pytree_node, "serialize": serialize, "cache": cache},
-        **kwargs,
-    )
-
-
-class CachedProperty:
-    """Sentinel attribute wrapper to signal that a method is a property
-    but must be cached.
-    """
-
-    def __init__(self, method, pytree_node=False):
-        self.name = method.__name__
-        self.cache_name = _cache_name(self.name)
-        self.method = method
-        self.pytree_node = pytree_node
-        self.type = method.__annotations__.get("return", MISSING)
-        self.doc = method.__doc__
-
-        if self.type is MISSING:
-            raise TypeError(
-                "Cached property {method} requires a return type annotation."
-            )
-
-    def __repr__(self):
-        return (
-            f"CachedProperty(name={self.name}, "
-            f"type={self.type}, pytree_node={self.pytree_node})"
-        )
-
-
-def property_cached(fun=None, pytree_node=False):
-    """Decorator to make the method behave as a property but cache the resulting value and
-    clears it upon replace.
-
-    Args:
-        pytree_node: a leaf node in the pytree representation of this dataclass.
-            If False this must be hashable
-    """
-    if fun is None:
-        return partial(property_cached, pytree_node=pytree_node)
-
-    return CachedProperty(fun, pytree_node=pytree_node)
-
-
 def _set_annotation(clz, attr, typ):
     if "__annotations__" not in clz.__dict__:
         setattr(clz, "__annotations__", {})
@@ -206,7 +149,7 @@ def process_cached_properties(clz, globals=None):
             body_lines,
             return_type=cp.type,
             globals=globals,
-            doc=cp.doc,
+            doc=cp.__doc__,
         )
         prop_fun = property(fun)
         setattr(clz, name, prop_fun)
@@ -215,7 +158,7 @@ def process_cached_properties(clz, globals=None):
     for b in clz.__mro__[1:]:
         # Only process classes that have been processed by our
         # decorator.  That is, they have a _FIELDS attribute.
-        for (name, cp) in getattr(b, _CACHES, {}).items():
+        for name, cp in getattr(b, _CACHES, {}).items():
             if name not in cached_props:
                 cached_props[name] = cp
 
@@ -271,7 +214,11 @@ def purge_cache_fields(clz):
                 flds.pop(cname)
 
 
-def attach_preprocess_init(data_clz, *, globals={}, init_doc=MISSING, cache_hash=False):
+def attach_preprocess_init(
+    data_clz, *, globals=None, init_doc=MISSING, cache_hash=False, is_pytree=False
+):
+    if globals is None:
+        globals = {}
 
     # If there is no __pre_init__ method in the class, create a default
     # one calling pre init on super() if there is one.
@@ -282,7 +229,6 @@ def attach_preprocess_init(data_clz, *, globals={}, init_doc=MISSING, cache_hash
                 args, kwargs = getattr(super(data_clz, self), _PRE_INIT_NAME)(
                     *args, **kwargs
                 )
-
             return args, kwargs
 
         _set_new_attribute(data_clz, _PRE_INIT_NAME, _preprocess_args_default)
@@ -298,6 +244,10 @@ def attach_preprocess_init(data_clz, *, globals={}, init_doc=MISSING, cache_hash
     body_lines = [
         "if not __skip_preprocess:",
         f"\targs, kwargs = {self_name}.{_PRE_INIT_NAME}(*args, **kwargs)",
+        "if True:" if is_pytree else "if False:",
+        "\t_args_pytree, _kwargs_pytree = kwargs['__base_init_args']",
+        "\tdel kwargs['__base_init_args']",
+        "\tsuper(data_class, self).__init__(*_args_pytree, **_kwargs_pytree)",
         f"{self_name}.{_DATACLASS_INIT_NAME}(*args, **kwargs)",
         "if __precompute_cached_properties:",
         f"\t{self_name}.{PRECOMPUTE_CACHED_PROPERTY_NAME}()",
@@ -309,6 +259,8 @@ def attach_preprocess_init(data_clz, *, globals={}, init_doc=MISSING, cache_hash
             f"BUILTINS.object.__setattr__({self_name},{_hash_cache_name(data_clz.__name__)!r},Uninitialized)"
         )
 
+    globals["data_class"] = data_clz
+
     fun = _create_fn(
         "__init__",
         [
@@ -328,11 +280,14 @@ def attach_preprocess_init(data_clz, *, globals={}, init_doc=MISSING, cache_hash
     setattr(data_clz, "__init__", fun)
 
 
-def replace_hash_method(data_clz, *, globals={}):
+def replace_hash_method(data_clz, *, globals=None):
     """
     Replace __hash__ by a method that checks if it has already been computed
     and returns the cached value otherwise.
     """
+    if globals is None:
+        globals = {}
+
     self_name = "self"
     hash_cache_name = _hash_cache_name(data_clz.__name__)
 
@@ -355,12 +310,39 @@ def replace_hash_method(data_clz, *, globals={}):
     setattr(data_clz, "__hash__", fun)
 
 
-def dataclass(clz=None, *, init_doc=MISSING, cache_hash=False, _frozen=True):
+_T = TypeVar("_T")
+
+
+@dataclass_transform(field_specifiers=(field,))  # type: ignore[literal-required]
+@overload
+def dataclass(
+    clz: _T, init_doc=MISSING, cache_hash: bool = False, _frozen: bool = True
+) -> _T: ...
+
+
+@dataclass_transform(field_specifiers=(field,))  # type: ignore[literal-required]
+@overload
+def dataclass(
+    init_doc=MISSING, cache_hash: bool = False, _frozen: bool = True
+) -> Callable[[_T], _T]: ...
+
+
+@dataclass_transform(field_specifiers=(field,))  # type: ignore[literal-required]
+def dataclass(
+    clz=None, *, init_doc=MISSING, cache_hash: bool = False, _frozen: bool = True
+):
     """
     Decorator creating a NetKet-flavour dataclass.
     This behaves as a flax dataclass, that is a Frozen python dataclass, with a twist!
     See their documentation for standard behaviour.
 
+    .. warning::
+
+        This decorator should be used together with classes inheriting from
+        :ref:`netket.utils.struct.Pytree`. While simple cases will work
+        for now, it is not guaranteed that the behaviour will be always correct
+        and stable.
+
     The new functionalities added by NetKet are:
      - it is possible to define a method `__pre_init__(*args, **kwargs) ->
        Tuple[Tuple,Dict]` that processes the arguments and keyword arguments provided
@@ -387,11 +369,58 @@ def dataclass(clz=None, *, init_doc=MISSING, cache_hash=False, _frozen=True):
         _frozen: (default True) controls whether the resulting class is frozen or not.
             If it is not frozen, extra care should be taken.
     """
-
+    # Support passing arguments to the decorator (e.g. @dataclass(kw_only=True))
     if clz is None:
-        return partial(
-            dataclass, init_doc=init_doc, cache_hash=cache_hash, _frozen=_frozen
-        )
+        return partial(dataclass, init_doc=init_doc, cache_hash=cache_hash, _frozen=_frozen)  # type: ignore[bad-return-type]
+
+    is_pytree = Pytree in clz.__mro__
+
+    if is_pytree:
+        if not (clz._pytree__class_is_mutable ^ _frozen):
+            raise TypeError(
+                f"Inheriting from a mutable={clz._pytree__class_is_mutable} but _frozen={_frozen}"
+            )
+        if clz._pytree__class_dynamic_nodes:
+            raise TypeError(
+                "dynamic nodes Pytrees are incompatible with the dataclass decorator."
+            )
+
+        # let the base class handle the frozeness
+        _frozen = False
+
+        if _PRE_INIT_NAME in clz.__dict__:
+            msg = f"""
+            You defined `__pre_init__(*args, **kwargs)` in a netket
+            dataclass (a class decorated with @nk.utils.struct.dataclass) which
+            inherits from a `nk.utils.struct.Pytree`.
+
+            The class is {clz}.
+
+            This behaviour is not supported and might break. Please remove
+            the decorator and just inherit from the base class, defining
+            a standard `__init__` method which calls `super().__init__(...)`
+            as usual.
+
+            If you need help, reach out with us.
+            """
+            warnings.warn(msg, category=FutureWarning, stacklevel=1)
+
+        if "__post_init__" in clz.__dict__:
+            msg = f"""
+            You defined `__post_init__(self)` in a netket
+            dataclass (a class decorated with @nk.utils.struct.dataclass) which
+            inherits from a `nk.utils.struct.Pytree`.
+
+            The class is {clz}.
+
+            This behaviour is not supported and might break. Please remove
+            the decorator and just inherit from the base class, defining
+            a standard `__init__` method which calls `super().__init__(...)`
+            as usual.
+
+            If you need help, reach out with us.
+            """
+            warnings.warn(msg, category=FutureWarning, stacklevel=1)
 
     # get globals of the class to put generated methods in there
     _globals = get_class_globals(clz)
@@ -400,23 +429,75 @@ def dataclass(clz=None, *, init_doc=MISSING, cache_hash=False, _frozen=True):
     process_cached_properties(clz, globals=_globals)
     # create the dataclass
     data_clz = dataclasses.dataclass(frozen=_frozen)(clz)
+
     purge_cache_fields(data_clz)
     # attach the custom preprocessing of init arguments
     attach_preprocess_init(
-        data_clz, globals=_globals, init_doc=init_doc, cache_hash=cache_hash
+        data_clz,
+        globals=_globals,
+        init_doc=init_doc,
+        cache_hash=cache_hash,
+        is_pytree=is_pytree,
     )
     if cache_hash:
         replace_hash_method(data_clz, globals=_globals)
 
+    # if it's an 'auto-style PyTree', use standard dataclass-logic
+    # and do not register it with jax/flax
+    if is_pytree:
+        # This is used in the `__pre_init__` of Pytrees to identify
+        # the last user-defined __init__ method, which will reside in
+        # the top-most non-dataclass class.
+        for clz in data_clz.__mro__:
+            if clz == Pytree:
+                n_args_max = 0
+                break
+            if not hasattr(clz, "__dataclass_params__"):
+                if "__init__" in clz.__dict__:
+                    n_args_max = maximum_positional_args(clz.__init__) - 1
+                    break
+        setattr(data_clz, DATACLASS_USER_INIT_N_ARGS, n_args_max)
+
+        # Forbid fields with same name as keyword arguments in the pytree below
+        pytree_arg_names = keyword_arg_names(clz.__init__)
+        _data_clz_fields = getattr(data_clz, _FIELDS)
+        args_not_ok = [nm for nm in pytree_arg_names if nm in _data_clz_fields]
+        if len(args_not_ok) > 0:
+            raise ValueError(
+                f"""
+                You cannot declare a dataclass with an attribute having the
+                same name as an argument to the `__init__` function of the
+                Pytree it inherits from.
+
+                Pytree {clz = } has the following argument names used in its
+                __init__ method: {pytree_arg_names}, and you cannot use them
+                as attributes or fields of this dataclass.
+
+                In your definition of the dataclass {data_clz = }, which inherits
+                from the pytree above, you have the following illegally named fields:
+                    {args_not_ok}
+
+                Rename them to a valid name to make this error disappear!
+
+                ====================================================================
+                Note: this is experimental functionality. If you believe this should
+                work, please open a bug report on the NetKet repository.
+                """
+            )
+
+        return data_clz
+
     # flax stuff: identify states
     meta_fields = []
     data_fields = []
-    for name, field_info in getattr(data_clz, _FIELDS, {}).items():
+
+    # for name, field_info in getattr(data_clz, _FIELDS, {}).items():
+    for field_info in dataclasses.fields(data_clz):
         is_pytree_node = field_info.metadata.get("pytree_node", True)
         if is_pytree_node:
-            data_fields.append(name)
+            data_fields.append(field_info.name)
         else:
-            meta_fields.append(name)
+            meta_fields.append(field_info.name)
 
     # List the cache fields
     cache_fields = []
@@ -440,9 +521,11 @@ def dataclass(clz=None, *, init_doc=MISSING, cache_hash=False, _frozen=True):
     data_clz.replace = replace
 
     # support for jax pytree flattening unflattening
-    def iterate_clz(x):
+    def iterate_clz_with_keys(x):
         meta = tuple(getattr(x, name) for name in meta_fields)
-        data = tuple(getattr(x, name) for name in data_fields)
+        data = tuple(
+            (jax.tree_util.GetAttrKey(name), getattr(x, name)) for name in data_fields
+        )
         return data, meta
 
     def clz_from_iterable(meta, data):
@@ -451,7 +534,9 @@ def dataclass(clz=None, *, init_doc=MISSING, cache_hash=False, _frozen=True):
         kwargs = dict(meta_args + data_args)
         return data_clz(__skip_preprocess=True, **kwargs)
 
-    jax.tree_util.register_pytree_node(data_clz, iterate_clz, clz_from_iterable)
+    jax.tree_util.register_pytree_with_keys(
+        data_clz, iterate_clz_with_keys, clz_from_iterable
+    )
 
     # flax serialization
     skip_serialize_fields = []
@@ -476,19 +561,26 @@ def dataclass(clz=None, *, init_doc=MISSING, cache_hash=False, _frozen=True):
                 if name not in state:
                     raise ValueError(
                         f"Missing field {name} in state dict while restoring"
-                        f" an instance of {clz.__name__}"
+                        f" an instance of {clz.__name__},"
+                        f" at path {serialization.current_path()}"
                     )
                 value = getattr(x, name)
                 value_state = state.pop(name)
-                updates[name] = serialization.from_state_dict(value, value_state)
+                updates[name] = serialization.from_state_dict(
+                    value, value_state, name=name
+                )
         if state:
             names = ",".join(state.keys())
             raise ValueError(
                 f'Unknown field(s) "{names}" in state dict while'
                 f" restoring an instance of {clz.__name__}"
+                f" at path {serialization.current_path()}"
             )
         return x.replace(**updates)
 
     serialization.register_serialization_state(data_clz, to_state_dict, from_state_dict)
 
-    return data_clz
+    # add a _netket_dataclass flag to distinguish from regular dataclasses
+    data_clz._netket_dataclass = True
+
+    return data_clz  # type: ignore
