@@ -13,7 +13,7 @@
 # limitations under the License.
 
 from functools import partial
-from typing import Callable
+from collections.abc import Callable
 import warnings
 
 import jax
@@ -27,10 +27,14 @@ from netket.utils.dispatch import dispatch
 from netket.operator import (
     AbstractOperator,
     DiscreteOperator,
+    DiscreteJaxOperator,
     ContinuousOperator,
     Squared,
 )
 
+# to move up once stabilized
+from netket.operator._abstract_observable import AbstractObservable
+
 from netket.vqs import expect
 
 from netket.vqs.mc import (
@@ -48,6 +52,13 @@ def get_local_kernel(vstate: MCState, Ô: Squared, chunk_size: int):  # noqa: F
     return kernels.local_value_squared_kernel_chunked
 
 
+@dispatch
+def get_local_kernel(  # noqa: F811
+    vstate: MCState, Ô: DiscreteJaxOperator, chunk_size: int
+):  # noqa: F811
+    return kernels.local_value_kernel_jax_chunked
+
+
 @dispatch
 def get_local_kernel(  # noqa: F811
     vstate: MCState, Ô: DiscreteOperator, chunk_size: int
@@ -55,31 +66,29 @@ def get_local_kernel(  # noqa: F811
     return kernels.local_value_kernel_chunked
 
 
-def _local_continuous_kernel(kernel, logpsi, pars, σ, args, *, chunk_size=None):
-    def _kernel(σ):
-        return kernel(logpsi, pars, σ, args)
-
-    return nkjax.apply_chunked(_kernel, in_axes=0, chunk_size=chunk_size)(σ)
+def _local_continuous_kernel(logpsi, pars, σ, op, *, chunk_size=None):
+    return nkjax.apply_chunked(
+        lambda x: op._expect_kernel(logpsi, pars, x), in_axes=0, chunk_size=chunk_size
+    )(σ)
 
 
 @dispatch
 def get_local_kernel(  # noqa: F811
     vstate: MCState, Ô: ContinuousOperator, chunk_size: int
 ):
-    return nkjax.HashablePartial(_local_continuous_kernel, Ô._expect_kernel)
+    return _local_continuous_kernel
 
 
-# If batch_size is None, ignore it and remove it from signature so that we fall back
-# to already implemented methods
+# If batch_size is unspecified, set it to None
 @expect.dispatch
-def expect_nochunking(vstate: MCState, operator: AbstractOperator, chunk_size: None):
-    return expect(vstate, operator)
+def expect_chunking_unspecified(vstate: MCState, operator: AbstractObservable):
+    return expect(vstate, operator, None)
 
 
 # if no implementation exists for batched, fall back to unbatched methods.
-@expect.dispatch
+@expect.dispatch(precedence=-10)
 def expect_fallback(
-    vstate: MCState, operator: AbstractOperator, chunk_size
+    vstate: MCState, operator: AbstractObservable, chunk_size: int | tuple
 ):  # noqa: F811
     warnings.warn(
         f"Ignoring chunk_size={chunk_size} for expect_and_grad method with signature "
@@ -87,7 +96,7 @@ def expect_fallback(
         f"chunking for this signature exists."
     )
 
-    return expect(vstate, operator)
+    return expect(vstate, operator, None)
 
 
 @expect.dispatch
