@@ -13,15 +13,14 @@
 # limitations under the License.
 
 from functools import partial
-from typing import Callable, Tuple
+from collections.abc import Callable
 
 import jax
 from jax import numpy as jnp
-from flax.core.scope import CollectionFilter
+from flax.core.scope import CollectionFilter, DenyList  # noqa: F401
 
 from netket import jax as nkjax
 from netket.stats import Stats, statistics
-from netket.utils import mpi
 from netket.utils.types import PyTree
 from netket.utils.dispatch import dispatch
 
@@ -41,9 +40,10 @@ from .state import MCState
 def expect_and_forces(  # noqa: F811
     vstate: MCState,
     Ô: AbstractOperator,
+    chunk_size: None,
     *,
-    mutable: CollectionFilter,
-) -> Tuple[Stats, PyTree]:
+    mutable: CollectionFilter = False,
+) -> tuple[Stats, PyTree]:
     σ, args = get_local_kernel_arguments(vstate, Ô)
 
     local_estimator_fun = get_local_kernel(vstate, Ô)
@@ -73,13 +73,12 @@ def forces_expect_hermitian(
     model_state: PyTree,
     σ: jnp.ndarray,
     local_value_args: PyTree,
-) -> Tuple[PyTree, PyTree]:
+) -> tuple[PyTree, PyTree]:
+    n_chains = σ.shape[0]
+    if σ.ndim >= 3:
+        σ = jax.lax.collapse(σ, 0, 2)
 
-    σ_shape = σ.shape
-    if jnp.ndim(σ) != 2:
-        σ = σ.reshape((-1, σ_shape[-1]))
-
-    n_samples = σ.shape[0] * mpi.n_nodes
+    n_samples = σ.shape[0]
 
     O_loc = local_value_kernel(
         model_apply_fun,
@@ -88,7 +87,7 @@ def forces_expect_hermitian(
         local_value_args,
     )
 
-    Ō = statistics(O_loc.reshape(σ_shape[:-1]).T)
+    Ō = statistics(O_loc.reshape((n_chains, -1)))
 
     O_loc -= Ō.mean
 
@@ -106,4 +105,8 @@ def forces_expect_hermitian(
 
     new_model_state = new_model_state[0] if is_mutable else None
 
-    return Ō, jax.tree_map(lambda x: mpi.mpi_sum_jax(x)[0], Ō_grad), new_model_state
+    return (
+        Ō,
+        Ō_grad,
+        new_model_state,
+    )
