@@ -13,18 +13,18 @@
 # limitations under the License.
 
 from functools import partial
-from typing import Any, Callable, Tuple
+from typing import Any
+from collections.abc import Callable
 import warnings
 
 import jax
 from jax import numpy as jnp
-from jax import tree_map
-from flax.core.scope import CollectionFilter
+from flax.core.scope import CollectionFilter, DenyList  # noqa: F401
 
 from netket import jax as nkjax
 from netket.operator import AbstractOperator
+from netket.operator._abstract_observable import AbstractObservable
 from netket.stats import Stats, statistics
-from netket.utils import mpi
 from netket.utils.types import PyTree
 
 from netket.vqs import expect_and_forces
@@ -36,23 +36,21 @@ from netket.vqs.mc import (
 from .state import MCState
 
 
-# If batch_size is None, ignore it and remove it from signature
+# If batch_size is unspecified, set it to None
 @expect_and_forces.dispatch
-def expect_and_forces_nochunking(  # noqa: F811
+def expect_and_forces_chunking_unspecified(  # noqa: F811
     vstate: MCState,
-    operator: AbstractOperator,
-    chunk_size: None,
-    *args,
+    operator: AbstractObservable,
     **kwargs,
 ):
-    return expect_and_forces(vstate, operator, *args, **kwargs)
+    return expect_and_forces(vstate, operator, None, **kwargs)
 
 
 # if no implementation exists for batched, run the code unbatched
-@expect_and_forces.dispatch
+@expect_and_forces.dispatch(precedence=-10)
 def expect_and_forces_fallback(  # noqa: F811
     vstate: MCState,
-    operator: AbstractOperator,
+    operator: AbstractObservable,
     chunk_size: Any,
     *args,
     **kwargs,
@@ -60,10 +58,11 @@ def expect_and_forces_fallback(  # noqa: F811
     warnings.warn(
         f"Ignoring chunk_size={chunk_size} for expect_and_forces method with signature "
         f"({type(vstate)}, {type(operator)}) because no implementation supporting "
-        f"chunking for this signature exists."
+        f"chunking for this signature exists.",
+        stacklevel=2,
     )
 
-    return expect_and_forces(vstate, operator, *args, **kwargs)
+    return expect_and_forces(vstate, operator, None, *args, **kwargs)
 
 
 @expect_and_forces.dispatch
@@ -72,8 +71,8 @@ def expect_and_forces_impl(  # noqa: F811
     Ô: AbstractOperator,
     chunk_size: int,
     *,
-    mutable: CollectionFilter,
-) -> Tuple[Stats, PyTree]:
+    mutable: CollectionFilter = False,
+) -> tuple[Stats, PyTree]:
     σ, args = get_local_kernel_arguments(vstate, Ô)
 
     local_estimator_fun = get_local_kernel(vstate, Ô, chunk_size)
@@ -105,13 +104,12 @@ def forces_expect_hermitian_chunked(
     model_state: PyTree,
     σ: jnp.ndarray,
     local_value_args: PyTree,
-) -> Tuple[PyTree, PyTree]:
-
+) -> tuple[PyTree, PyTree]:
     σ_shape = σ.shape
     if jnp.ndim(σ) != 2:
         σ = σ.reshape((-1, σ_shape[-1]))
 
-    n_samples = σ.shape[0] * mpi.n_nodes
+    n_samples = σ.shape[0]
 
     O_loc = local_value_kernel_chunked(
         model_apply_fun,
@@ -121,7 +119,7 @@ def forces_expect_hermitian_chunked(
         chunk_size=chunk_size,
     )
 
-    Ō = statistics(O_loc.reshape(σ_shape[:-1]).T)
+    Ō = statistics(O_loc.reshape(σ_shape[:-1]))
 
     O_loc -= Ō.mean
 
@@ -142,8 +140,7 @@ def forces_expect_hermitian_chunked(
     else:
         raise NotImplementedError
 
-    Ō_grad = vjp_fun_chunked(
+    (Ō_grad,) = vjp_fun_chunked(
         (jnp.conjugate(O_loc) / n_samples),
-    )[0]
-
-    return Ō, tree_map(lambda x: mpi.mpi_sum_jax(x)[0], Ō_grad), new_model_state
+    )
+    return Ō, Ō_grad, new_model_state
