@@ -13,84 +13,83 @@
 # limitations under the License.
 
 from functools import partial
-from typing import Callable, Tuple
+from collections.abc import Callable
 
 import jax
 from jax import numpy as jnp
-from flax.core.scope import CollectionFilter
+from flax.core.scope import CollectionFilter, DenyList  # noqa: F401
 
 from netket import jax as nkjax
 from netket import config
 from netket.stats import Stats
-from netket.utils import mpi
+from netket.utils import dispatch
 from netket.utils.types import PyTree
-from netket.utils.dispatch import TrueT, FalseT
 
 from netket.operator import (
     AbstractOperator,
-    DiscreteOperator,
     Squared,
 )
 
 from netket.vqs import expect_and_grad, expect_and_forces
 
-from netket.vqs.mc import (
-    get_local_kernel_arguments,
-    get_local_kernel,
-)
+from ..common import force_to_grad, get_local_kernel_arguments, get_local_kernel
 
 from .state import MCState
 
 
-# Implementation of expect_and_grad for `use_covariance == True` (due to the TrueT
-# type in the signature).` This case is equivalent to the composition of the
-# `expect_and_forces` and `_force_to_grad` functions.
+# General implementation checking hermitianity
 @expect_and_grad.dispatch
-def expect_and_grad_covariance(
+def expect_and_grad_default_formula(
     vstate: MCState,
     Ô: AbstractOperator,
-    use_covariance: TrueT,
-    *,
-    mutable: CollectionFilter,
-) -> Tuple[Stats, PyTree]:
-    Ō, Ō_grad = expect_and_forces(vstate, Ô, mutable=mutable)
-    Ō_grad = _force_to_grad(Ō_grad, vstate.parameters)
-    return Ō, Ō_grad
+    chunk_size: int | None,
+    *args,
+    mutable: CollectionFilter = False,
+    use_covariance: bool | None = None,
+) -> tuple[Stats, PyTree]:
+    if use_covariance is None:
+        use_covariance = Ô.is_hermitian
+
+    if use_covariance:
+        # Implementation of expect_and_grad for `use_covariance == True` (due to the Literal[True]
+        # type in the signature).` This case is equivalent to the composition of the
+        # `expect_and_forces` and `force_to_grad` functions.
+        # return expect_and_grad_from_covariance(vstate, Ô, *args, mutable=mutable)
+        Ō, Ō_grad = expect_and_forces(vstate, Ô, chunk_size, *args, mutable=mutable)
+        Ō_grad = force_to_grad(Ō_grad, vstate.parameters)
+        return Ō, Ō_grad
+    else:
+        return expect_and_grad_nonhermitian(
+            vstate, Ô, chunk_size, *args, mutable=mutable
+        )
 
 
-@jax.jit
-def _force_to_grad(Ō_grad, parameters):
-    """
-    Converts the forces vector F_k = cov(O_k, E_loc) to the observable gradient.
-    In case of a complex target (which we assume to correspond to a holomorphic
-    parametrization), this is the identity. For real-valued parameters, the gradient
-    is 2 Re[F].
-    """
-    Ō_grad = jax.tree_map(
-        lambda x, target: (x if jnp.iscomplexobj(target) else 2 * x.real).astype(
-            target.dtype
-        ),
-        Ō_grad,
-        parameters,
-    )
-    return Ō_grad
+# Squared is a special operator...
+@expect_and_grad.dispatch
+def expect_and_grad_squared_op(
+    vstate: MCState,
+    Ô: Squared,
+    chunk_size: int | None,
+    *args,
+    mutable: CollectionFilter = False,
+    use_covariance: bool | None = None,
+) -> tuple[Stats, PyTree]:
+    if use_covariance is not None:
+        raise ValueError(
+            "Cannot specify `use_covariance` with Squared[...] operator.\n"
+            "This operator must use the same formula as non-hermitian operators to work."
+        )
+    return expect_and_grad_nonhermitian(vstate, Ô, chunk_size, *args, mutable=mutable)
 
 
-# Specialized dispatch rule for pure states with squared operators as well as general operators
-# with use_covariance == False (experimental).
-@expect_and_grad.dispatch_multi(
-    (MCState, Squared[DiscreteOperator], FalseT),
-    (MCState, Squared[AbstractOperator], FalseT),
-    (MCState, AbstractOperator, FalseT),
-)
-def expect_and_grad_nonherm(
-    vstate,
+@dispatch.dispatch
+def expect_and_grad_nonhermitian(
+    vstate: MCState,
     Ô,
-    use_covariance,
+    chunk_size: None,
     *,
-    mutable: CollectionFilter,
-) -> Tuple[Stats, PyTree]:
-
+    mutable: CollectionFilter = False,
+):
     if not isinstance(Ô, Squared) and not config.netket_experimental:
         raise RuntimeError(
             """
@@ -107,11 +106,11 @@ def expect_and_grad_nonherm(
 
     local_estimator_fun = get_local_kernel(vstate, Ô)
 
-    Ō, Ō_grad, new_model_state = grad_expect_operator_kernel(
+    Ō, Ō_grad, new_model_state = _grad_expect_nonherm_kernel(
         local_estimator_fun,
         vstate._apply_fun,
-        vstate.sampler.machine_pow,
         mutable,
+        vstate.sampler.machine_pow,
         vstate.parameters,
         vstate.model_state,
         σ,
@@ -124,21 +123,20 @@ def expect_and_grad_nonherm(
     return Ō, Ō_grad
 
 
-@partial(jax.jit, static_argnums=(0, 1, 2, 3))
-def grad_expect_operator_kernel(
+@partial(jax.jit, static_argnums=(0, 1, 2))
+def _grad_expect_nonherm_kernel(
     local_value_kernel: Callable,
     model_apply_fun: Callable,
-    machine_pow: int,
     mutable: CollectionFilter,
+    machine_pow: float,
     parameters: PyTree,
     model_state: PyTree,
     σ: jnp.ndarray,
     local_value_args: PyTree,
-) -> Tuple[PyTree, PyTree, Stats]:
-
-    σ_shape = σ.shape
-    if jnp.ndim(σ) != 2:
-        σ = σ.reshape((-1, σ_shape[-1]))
+) -> tuple[PyTree, PyTree, Stats]:
+    n_chains = σ.shape[0]
+    if σ.ndim >= 3:
+        σ = jax.lax.collapse(σ, 0, 2)
 
     is_mutable = mutable is not False
     logpsi = lambda w, σ: model_apply_fun(
@@ -155,7 +153,7 @@ def grad_expect_operator_kernel(
             pars,
             σ,
             local_value_args,
-            n_chains=σ_shape[0],
+            n_chains=n_chains,
         )
 
     Ō, Ō_pb, Ō_stats = nkjax.vjp(
@@ -163,14 +161,6 @@ def grad_expect_operator_kernel(
     )
     Ō_pars_grad = Ō_pb(jnp.ones_like(Ō))[0]
 
-    # This term below is needed otherwise it does not match the value obtained by
-    # (ha@ha).collect(). I'm unsure of why it is needed.
-    Ō_pars_grad = jax.tree_map(
-        lambda x, target: x / 2 if jnp.iscomplexobj(target) else x,
-        Ō_pars_grad,
-        parameters,
-    )
-
     if is_mutable:
         raise NotImplementedError(
             "gradient of non-hermitian operators over mutable models "
@@ -180,6 +170,6 @@ def grad_expect_operator_kernel(
 
     return (
         Ō_stats,
-        jax.tree_map(lambda x: mpi.mpi_mean_jax(x)[0], Ō_pars_grad),
+        Ō_pars_grad,
         new_model_state,
     )
